/*
 *
 * PAM auth functions for pam_cifs.
 *   Copyright Alexander O. Yuriev, 1996.  All rights reserved.
 *   NIS+ support by Thorsten Kukuk <kukuk@weber.uni-paderborn.de>
 *   Copyright Jan Rkorajski, 1999.  All rights reserved.
 *   Copyright (c) International Business Machines  Corp., 2001
 *     IBM Contact: RCHNSSPT@us.ibm.com
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/* #define DEBUG */

#ifdef linux
#define _GNU_SOURCE
#include <features.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>

/* indicate the following groups are defined */

#define PAM_SM_AUTH
#define _PAM_EXTERN_FUNCTIONS
#include <security/_pam_macros.h>
#include <security/pam_modules.h>

#ifndef LINUX_PAM
#include <security/pam_appl.h>
#endif              /* LINUX_PAM */

#include <pwd.h>
#include "support.h"
#include "pam_cifs_common.h"
#include "pam_cifs_config.h"
#include "pam_cifs_user.h"
#include "pam_cifs_util.h"


/*
 * PAM framework looks for these entry-points to pass control to the
 * authentication module.
 */

/*
 * pam_sm_authenticate() performs CIFS authentication.
 */

#define _CIFS_AUTHTOK  "-CIFS-PASS"

#define AUTH_RETURN                     \
{                               \
    if (on(CIFS_LIKE_AUTH, ctrl) && ret_data) {     \
        D(("recording return code for next time [%d]",  \
                    retval));       \
        pam_set_data(pamh, "cifs_setcred_return",   \
                (void *) retval, NULL);         \
    }                           \
    D(("done. [%s]", pam_strerror(pamh, retval)));      \
    return retval;                      \
}

PAM_EXTERN int pam_sm_authenticate(pam_handle_t * pamh, int flags
                   ,int argc, const char **argv)
{
    unsigned int ctrl;
    int retval, *ret_data = NULL;
    char *authServer = NULL;
    const char *name, *p;
    struct passwd *pwent;
    struct passwd user_entry;
    struct stat fileInfo;
    char homedir[CIFS_USER_MAX_PATH_LEN];
    int rc;

    D(("called."));

    ctrl = _set_ctrl(pamh, flags, NULL, argc, argv);

    /* Get a few bytes so we can pass our return value to
       pam_sm_setcred(). */
    ret_data = malloc(sizeof(int));

    /* get the user'name' */
    retval = pam_get_user(pamh, &name, "User Name: ");
    if (retval == PAM_SUCCESS) {
        /*
         * Various libraries at various times have had bugs related to
         * '+' or '-' as the first character of a user name. Don't take
         * any chances here. Require that the username starts with an
         * alphanumeric character.
         */

        if (name == NULL || !isalnum(*name)) {
            _log_err(LOG_ERR, pamh, "bad username [%s]", name);
            retval = PAM_USER_UNKNOWN;
            AUTH_RETURN
        }
        if (retval == PAM_SUCCESS && on(CIFS_DEBUG, ctrl))
            D(("username [%s] obtained", name));
    } else {
        D(("trouble reading username"));
        if (retval == PAM_CONV_AGAIN) {
            D(("pam_get_user/conv() function is not ready yet"));
            /* it is safe to resume this function so we translate this
             * retval to the value that indicates we're happy to resume.
             */
            retval = PAM_INCOMPLETE;
        }
        AUTH_RETURN
    }

    /* Check for local user. */
    pwent = getpwnam(name);
    if (pwent != NULL) {

        /* User is in local database. */
        if (strstr(pwent->pw_gecos, CIFS_USER_CIFS_DESC_KEY) == NULL) {
            /* Description indicates that it is not a Windows user,
             * so cannot authenticate it.
             */
            retval = PAM_AUTH_ERR;
            AUTH_RETURN 
        }

    }
    
    /* User is not in local database yet, or is in local database
     * but description indicates that it is a Windows user.
     * Need to check the server for this user.
     */

    /* get this user's authentication token */
    retval = _cifs_read_password(pamh, ctrl, NULL, "Password: ", NULL
                     ,_CIFS_AUTHTOK, &p);
    if (retval != PAM_SUCCESS) {
        if (retval != PAM_CONV_AGAIN) {
            _log_err(LOG_CRIT, pamh, "auth could not identify password for [%s]"
                 ,name);
        } else {
            D(("conversation function is not ready yet"));
            /*
             * it is safe to resume this function so we translate this
             * retval to the value that indicates we're happy to resume.
             */
            retval = PAM_INCOMPLETE;
        }
        name = NULL;
        AUTH_RETURN
    }
    D(("user=%s, password=[%s]", name, p));

    /* At this point, this CIFS user name is required to have been typed in
     * all lower case to maintain consistency with names and directories.
     * This is due to the fact that CIFS authentication is case insensitive
     * to user names.  Reject the CIFS login if name is not lowercase. */
    if (!cifsIsLower(name))
    {
        _log_err(LOG_ERR, pamh, 
                 "Login rejected, CIFS username %s is not all lowercase letters", name);
        retval = PAM_AUTH_ERR;
        name=NULL;
        p = NULL;
        AUTH_RETURN 
    }

    /* Read pam_cifs.conf parameters */
    rc = initConfig(pamh, CIFS_CONFIG_PATH);
    if (rc != CONFIG_ERROR_SUCCESS)
    {
       /* Could not read config file, log error and return */
       _log_err(LOG_ERR, pamh, "Error reading CIFS config file %s, rc=%d\n", 
                CIFS_CONFIG_PATH, rc);
       retval = PAM_AUTH_ERR;
       AUTH_RETURN 
    }

#if 0
    printf("Parm table:\n");
    /* Print parm table */
    for(rc=0; (parmTable[rc].keyword != NULL); rc++)
    {
        printf("    %s=%s\n", parmTable[rc].keyword, parmTable[rc].value);
    }
#endif

    /* Verify the password of this user */
    authServer = parmTable[PARSE_KEY_AUTH_SERVER].value;
    rc = _cifs_authenticate_user(parmTable[PARSE_KEY_DOMAIN].value,
                                  authServer,
                                  name, p);

    /* Log error if primary server was not available */
    if (rc == CIFS_UTIL_NOT_AVAILABLE_ERROR)
    {
        printf("Unable to connect to primary server %s, trying alternate.\n",
               authServer);

        _log_err(LOG_ERR, pamh, 
                 "Unable to connect to primary server %s, trying alternate",
                 authServer);
    }

    /* If server not available and alternate specified, try it */
    if((rc == CIFS_UTIL_NOT_AVAILABLE_ERROR) && 
       (parmTable[PARSE_KEY_AUTH_SERVER_ALT].value[0] != '\0'))
    {
        /* Try alternate server */
        authServer = parmTable[PARSE_KEY_AUTH_SERVER_ALT].value;
        rc = _cifs_authenticate_user(parmTable[PARSE_KEY_DOMAIN].value,
                                 authServer,
                                 name, p);
        if (rc == CIFS_UTIL_NOT_AVAILABLE_ERROR)
        {
            printf("Unable connect to secondary server %s.\n", authServer);
            _log_err(LOG_ERR, pamh, "Unable connect to secondary server %s",
                     authServer);
        }
    }

    /* Return auth error if not successful and not expired password */
    if( (rc != CIFS_UTIL_SUCCESS) && 
        (rc != CIFS_UTIL_PASSWORD_EXPIRED_ERROR))
    {
        _log_err(LOG_ERR, pamh, 
                 "CIFS authentication to server %s for user %s failed, rc=%d",
                 authServer, name, rc);
        retval = PAM_AUTH_ERR;
        goto EXIT;
    }

    if ( rc == CIFS_UTIL_PASSWORD_EXPIRED_ERROR )
    {
        /* Authentication was successful, but password must be changed.
         * Create a PAM data item to indicate this.  It will be detected
         * in the account management call */
        pam_set_data(pamh, CIFS_PAM_DATA_PASSWORD_EXPIRED, NULL, NULL);
    }

    retval = PAM_SUCCESS;

    /* Create user in the local cache passwd file if it does not exist */
    if(pwent == NULL)
    {
        /* User was valid and an entry for it does not yet 
         * exist. Create an entry for the user in the
         * local cache passwd file for CIFS users, so
         * that getpwnam() will work correctly.
         */ 
        user_entry.pw_name = (char *)name;
        user_entry.pw_passwd = "*";
        user_entry.pw_uid = atoi(parmTable[PARSE_KEY_COMMON_UID].value);
        user_entry.pw_gid = atoi(parmTable[PARSE_KEY_COMMON_GID].value); 
        user_entry.pw_gecos = CIFS_USER_DESC;
        strcpy(homedir, CIFS_USER_HOMEDIR_PATH);
        strncat(homedir, name, sizeof(homedir) - sizeof(CIFS_USER_HOMEDIR_PATH));
        homedir[sizeof(homedir) - 1] = '\0'; 
        user_entry.pw_dir = homedir; 
        user_entry.pw_shell = CIFS_USER_DEFAULT_SHELL;

        pwent = &user_entry;
 
        rc = append_user(&user_entry);
        if (rc != CIFS_USER_ERROR_SUCCESS) {
           /* User was not created successfully. */
           _log_err(LOG_ERR, pamh,
                    "auth could not create user entry for [%s], "
                    "error code was [%d]", name, rc);
           retval = PAM_AUTH_ERR;
        }
    }

    /* Create user's home directory if it does not exist */
    if (pwent->pw_dir[0] != '\0')
    {
        rc = stat(pwent->pw_dir, &fileInfo);
        if ((rc != 0) && (errno == ENOENT))
        {
            char cmdStr[128];

            /* printf("Creating user home dir %s\n", pwent->pw_dir); */
        
            /* Copy the skeleton directory */
            /* The format for pw_dir field is limited in width so that the
               temporary buffer is not overwritten */
            sprintf(cmdStr, CIFS_CONFIG_COPY_SKEL_CMD, pwent->pw_dir);
            rc = system(cmdStr);
            /* printf("cp rc=%d, cmd=%s\n", rc, cmdStr); */
            if (rc != 0)
            {
                _log_err(LOG_ERR, pamh, 
                         "Error copying skeleton directory to %s, rc=%d\n", 
                         pwent->pw_dir, rc);
            }

            /* Fix up permissions on user's top level home dir */
            rc= chmod(pwent->pw_dir, (S_IRUSR | S_IWUSR | S_IXUSR));
            if (rc != 0)
            {
                _log_err(LOG_ERR, pamh, 
                         "Error setting permissions on home directory %s, rc=%d\n", 
                         pwent->pw_dir, rc);
            }
            /* printf("chmod rc=%d\n", rc); */


            /* Fix up owner and group on each file/dir */
            sprintf(cmdStr, CIFS_CONFIG_SET_OWNER_CMD, pwent->pw_uid,
                    pwent->pw_gid, pwent->pw_dir);
            rc = system(cmdStr);
            /* printf("chown rc=%d, cmd=%s\n", rc, cmdStr); */
            if (rc != 0)
            {
                _log_err(LOG_ERR, pamh, 
                  "Error setting ownership on user's home directory %s, rc=%d\n",
                  pwent->pw_dir, rc);
            }
        }
    }
  
EXIT:  
    name = p = NULL;

    AUTH_RETURN
}


/* 
 * pam_sm_setcred() returns success.
 */

PAM_EXTERN int pam_sm_setcred(pam_handle_t * pamh, int flags
                  ,int argc, const char **argv)
{
    return PAM_SUCCESS;
}

#ifdef PAM_STATIC
struct pam_module _pam_cifs_auth_modstruct = {
    "pam_cifs_auth",
    pam_sm_authenticate,
    pam_sm_setcred,
    NULL,
    NULL,
    NULL,
    NULL,
};
#endif
