/*
 *
 * PAM passwd functions for pam_cifs.
 *   Main coding by Elliot Lee <sopwith@redhat.com>, Red Hat Software. 
 *   Copyright (C) 1996.
 *   Copyright (c) Jan Rkorajski, 1999.
 *   Copyright (c) International Business Machines  Corp., 2001
 *     IBM Contact: RCHNSSPT@us.ibm.com
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <security/_pam_aconf.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <malloc.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <pwd.h>
#include <syslog.h>
#include <shadow.h>
#include <time.h>       /* for time() */
#include <fcntl.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <rpc/rpc.h>
#include <rpcsvc/yp_prot.h>
#include <rpcsvc/ypclnt.h>

#ifdef USE_CRACKLIB
#include <crack.h>
#endif

#include <security/_pam_macros.h>

/* indicate the following groups are defined */

#define PAM_SM_PASSWORD

#include <security/pam_modules.h>

#ifndef LINUX_PAM
#include <security/pam_appl.h>
#endif              /* LINUX_PAM */

#include "support.h"
#include "pam_cifs_util.h"
#include "pam_cifs_config.h"
#include "pam_cifs_user.h"
#include "pam_cifs_common.h"

#if !((__GLIBC__ == 2) && (__GLIBC_MINOR__ >= 1))
extern int getrpcport(const char *host, unsigned long prognum,
              unsigned long versnum, unsigned int proto);
#endif              /* GNU libc 2.1 */

/*
 * PAM framework looks for these entry-points to pass control to the
 * password changing module.
 */

/*
   How it works:
   Gets in username (has to be done) from the calling program
   Does authentication of user (only if we are not running as root)
   Gets new password/checks for sanity
   Sets it.
 */

/* passwd/salt conversion macros */

#define ascii_to_bin(c) ((c)>='a'?(c-59):(c)>='A'?((c)-53):(c)-'.')
#define bin_to_ascii(c) ((c)>=38?((c)-38+'a'):(c)>=12?((c)-12+'A'):(c)+'.')

/* data tokens */

#define _CIFS_OLD_AUTHTOK   "-CIFS-OLD-PASS"
#define _CIFS_NEW_AUTHTOK   "-CIFS-NEW-PASS"

#define MAX_PASSWD_TRIES    3
#define PW_TMPFILE      "/etc/npasswd"
#define SH_TMPFILE      "/etc/nshadow"
#define CRACKLIB_DICTS      "/usr/share/dict/cracklib_dict"
#define OPW_TMPFILE     "/etc/security/nopasswd"
#define OLD_PASSWORDS_FILE  "/etc/security/opasswd"


PAM_EXTERN int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
                int argc, const char **argv)
{
    unsigned int ctrl, lctrl;
    int retval;
    int rc;
    int remember = -1;
    char *authServer = NULL;

    /* <DO NOT free() THESE> */
    const char *user;
    char *pass_old, *pass_new;
    /* </DO NOT free() THESE> */

    struct passwd *pwent;

    D(("called."));

    ctrl = _set_ctrl(pamh, flags, &remember, argc, argv);

    /* Read pam_cifs.conf parameters */
    rc = initConfig(pamh, CIFS_CONFIG_PATH);
    if (rc != CONFIG_ERROR_SUCCESS)
    {
       /* Could not read config file, log error and return */
       _log_err(LOG_ERR, pamh, "Error reading CIFS config file %s, rc=%d\n", 
                CIFS_CONFIG_PATH, rc);
       return PAM_AUTHTOK_ERR;
    }

#if 0
    printf("Parm table:\n");
    /* Print parm table */
    for(rc=0; (parmTable[rc].keyword != NULL); i++)
    {
        printf("    %s=%s\n", parmTable[rc].keyword, parmTable[rc].value);
    }
#endif

    /*
     * First get the name of a user
     */
    retval = pam_get_user(pamh, &user, "Username: ");
    if (retval == PAM_SUCCESS) {
        /*
         * Various libraries at various times have had bugs related to
         * '+' or '-' as the first character of a user name. Don't take
         * any chances here. Require that the username starts with an
         * alphanumeric character.
         */
        if (user == NULL || !isalnum(*user)) {
            _log_err(LOG_ERR, pamh, "bad username [%s]", user);
            return PAM_USER_UNKNOWN;
        }
        
        if (on(CIFS_DEBUG, ctrl))
            _log_err(LOG_DEBUG, pamh, "username [%s] obtained",
                     user);
    } else {
        if (on(CIFS_DEBUG, ctrl))
            _log_err(LOG_DEBUG, pamh,
                     "password - could not identify user");
        return retval;
    }

    /* Check for local user. */
    pwent = getpwnam(user);
    if (pwent != NULL) {
        /* User is in local database. */
        if (strstr(pwent->pw_gecos, CIFS_USER_CIFS_DESC_KEY) == NULL) {
            /* Description indicates that it is not a Windows user,
             * so cannot authenticate it.
             */
            return PAM_AUTHTOK_ERR; 
        }
    }
    
    /* User is not in local database yet, or is in local database
     * but description indicates that it is a Windows user.
     * Need to check the server for this user.
     */

    /*
     * This is not an AUTH module!
     */
    if (on(CIFS__NONULL, ctrl))
        set(CIFS__NULLOK, ctrl);

    if (on(CIFS__PRELIM, ctrl)) {
        /*
         * obtain and verify the current password (OLDAUTHTOK) for
         * the user.
         */
        char *Announce;

        D(("prelim check"));

        if (off(CIFS__IAMROOT, ctrl)) {

            /* instruct user what is happening */
#define greeting "Changing password for "
            Announce = (char *) malloc(sizeof(greeting) + strlen(user));
            if (Announce == NULL) {
                _log_err(LOG_CRIT, pamh,
                         "password - out of memory");
                return PAM_BUF_ERR;
            }
            (void) strcpy(Announce, greeting);
            (void) strcpy(Announce + sizeof(greeting) - 1, user);
#undef greeting

            lctrl = ctrl;
            set(CIFS__OLD_PASSWD, lctrl);
            retval = _cifs_read_password(pamh, lctrl
                             ,Announce
                             ,"Enter current password: "
                             ,NULL
                             ,_CIFS_OLD_AUTHTOK
                         ,(const char **) &pass_old);
            free(Announce);

            if (retval != PAM_SUCCESS) {
                _log_err(LOG_NOTICE, pamh
                 ,"password - (old) token not obtained");
                return retval;
            }

            /* At this point, this CIFS user name is required to have been typed in
             * all lower case to maintain consistency with names and directories.
             * This is due to the fact that CIFS authentication is case insensitive
             * to user names.  Reject the CIFS authentication if name is not lowercase. */
            if (!cifsIsLower(user))
            {
                _log_err(LOG_ERR, pamh, 
                    "Password change rejected, CIFS username %s is not all lowercase letters", 
                    user);
                return PAM_AUTHTOK_ERR;
            }

            /* verify that this is the password for this user */
            authServer = parmTable[PARSE_KEY_AUTH_SERVER].value;
            rc = _cifs_authenticate_user(parmTable[PARSE_KEY_DOMAIN].value,
                                         authServer,
                                         user, pass_old);
            
            /* Log error if primary server was not available */
            if (rc == CIFS_UTIL_NOT_AVAILABLE_ERROR)
            {
                printf("Unable to connect to primary server %s, trying alternate.\n",
                       authServer);

                _log_err(LOG_ERR, pamh,
                         "Unable to connect to primary server %s, trying alternate",
                         authServer);
            }

            /* If server not available and alternate specified, try it */
            if ((rc == CIFS_UTIL_NOT_AVAILABLE_ERROR) &&
                (parmTable[PARSE_KEY_AUTH_SERVER_ALT].value[0] != '\0'))
            {
                /* Try alternate server */
                authServer = parmTable[PARSE_KEY_AUTH_SERVER_ALT].value;
                rc = _cifs_authenticate_user(
                            parmTable[PARSE_KEY_DOMAIN].value, authServer,
                            user, pass_old);
                if (rc == CIFS_UTIL_NOT_AVAILABLE_ERROR)
                {
                    printf("Unable to connect to secondary server %s.\n", authServer);

                    _log_err(LOG_ERR, pamh, 
                        "Unable to connect to secondary server %s", authServer);
                }
    
            }

            /* Return auth error if not successful and not expired password */
            if( (rc != CIFS_UTIL_SUCCESS) && 
                (rc != CIFS_UTIL_PASSWORD_EXPIRED_ERROR))
            {
                _log_err(LOG_ERR, pamh, 
                         "CIFS authentication to server %s for user %s failed, rc=%d",
                         authServer, user, rc);
                retval = PAM_AUTHTOK_ERR;
            }
            else 
            {
                char *tempBuf;
                /* Allocate buffer to store server name that we just 
                 * authenticated with so it can be used in the update
                 * pass */
                tempBuf = malloc(strlen(authServer)+1);
                if (tempBuf != NULL)
                {
                    /* Save auth server we are using in pam data area
                     * Note??: Should really set up a call back to free this
                     * data just in case.  Buffer is normally freed in the
                     * update pass though. */
                    strcpy(tempBuf, authServer);
                    pam_set_data(pamh, CIFS_PAM_DATA_AUTH_SERVER, tempBuf, NULL);
                }

                retval = PAM_SUCCESS;
            }
            
        } else {
            D(("process run by root so do nothing this time around"));
            pass_old = NULL;
            retval = PAM_SUCCESS;   /* root doesn't have too */
        }

        if (retval != PAM_SUCCESS) {
            D(("Authentication failed"));
            pass_old = NULL;
            return retval;
        }
        retval = pam_set_item(pamh, PAM_OLDAUTHTOK, (const void *) pass_old);
        pass_old = NULL;
        if (retval != PAM_SUCCESS) {
            _log_err(LOG_CRIT, pamh,
                     "failed to set PAM_OLDAUTHTOK");
        }

    } else if (on(CIFS__UPDATE, ctrl)) {

        int retry = 0;

        /*
         * obtain the proposed password
         */

        D(("do update"));
        /*
         * get the old token back. NULL was ok only if root [at this
         * point we assume that this has already been enforced on a
         * previous call to this function].
         */

        if (off(CIFS_NOT_SET_PASS, ctrl)) {
            retval = pam_get_item(pamh, PAM_OLDAUTHTOK
                          ,(const void **) &pass_old);
        } else {
            retval = pam_get_data(pamh, _CIFS_OLD_AUTHTOK
                          ,(const void **) &pass_old);
            if (retval == PAM_NO_MODULE_DATA) {
                retval = PAM_SUCCESS;
                pass_old = NULL;
            }
        }
        D(("pass_old [%s]", pass_old));

        if (retval != PAM_SUCCESS) {
            _log_err(LOG_NOTICE, pamh, "user not authenticated");
            return retval;
        }

        D(("get new password now"));

        lctrl = ctrl;

        if (on(CIFS_USE_AUTHTOK, lctrl)) {
            set(CIFS_USE_FIRST_PASS, lctrl);
        }
        retry = 0;
        retval = PAM_AUTHTOK_ERR;
        while ((retval != PAM_SUCCESS) && (retry++ < MAX_PASSWD_TRIES)) {
            /*
             * use_authtok is to force the use of a previously entered
             * password -- needed for pluggable password strength checking
             */

            retval = _cifs_read_password(pamh, lctrl
                             ,NULL
                             ,"Enter new password: "
                             ,"Retype new password: "
                             ,_CIFS_NEW_AUTHTOK
                         ,(const char **) &pass_new);

            if (retval != PAM_SUCCESS) {
                if (on(CIFS_DEBUG, ctrl)) {
                    _log_err(LOG_ALERT, pamh
                         ,"password - new password not obtained");
                }
                pass_old = NULL;    /* tidy up */
                return retval;
            }
            D(("returned to _cifs_chauthtok"));

            /*
             * At this point we know who the user is and what they
             * propose as their new password. Verify that the new
             * password is acceptable.
             */

            if (pass_new[0] == '\0') {  /* "\0" password = NULL */
                pass_new = NULL;
            }
        }

        if (retval != PAM_SUCCESS) {
            _log_err(LOG_NOTICE, pamh,
                     "new password not acceptable");
            _pam_overwrite(pass_new);
            _pam_overwrite(pass_old);
            pass_new = pass_old = NULL; /* tidy up */
            return retval;
        }
        /*
         * By reaching here we have approved the passwords and must now
         * rebuild the password database file.
         */

        D(("new password obtained"));

        /* Get auth server name that was used in preliminary pass */
        authServer = NULL;
        rc = pam_get_data(pamh, CIFS_PAM_DATA_AUTH_SERVER, (const void**)&authServer);
        if((rc == PAM_SUCCESS) && (authServer != NULL))
        {
            rc = _cifs_change_password(authServer, user, pass_old, pass_new); 
            if (CIFS_UTIL_SUCCESS == rc)
            {
                retval = PAM_SUCCESS;
            }
            else 
            {
                _log_err(LOG_ERR, pamh, "Change password failed for %s, rc=%d",
                         user, rc);
                /* printf("Change password failed rc=%d, server=%s, user=%s, opw=%s, npw=%s\n",
                        rc, authServer, user, pass_old, pass_new); */
                retval = PAM_AUTHTOK_ERR;
            }
    
            /* Free up auth server buffer used in pam data */
            pam_set_data(pamh, CIFS_PAM_DATA_AUTH_SERVER, NULL, NULL);
            free(authServer);
        }
        else
        {
            /* Could not find saved auth server, something must have failed */
            /* in the primary pass */
            retval = PAM_AUTHTOK_ERR;
        }

        /* erbjor??? Log something here if failed? */
        _pam_overwrite(pass_new);
        _pam_overwrite(pass_old);
        pass_old = pass_new = NULL;
    } else {        /* something has broken with the module */
        _log_err(LOG_ALERT, pamh,
                 "password received unknown request");
        retval = PAM_ABORT;
    }

    D(("retval was %d", retval));
    return retval;
}


/* static module data */
#ifdef PAM_STATIC
struct pam_module _pam_cifs_passwd_modstruct = {
    "pam_cifs_passwd",
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    pam_sm_chauthtok,
};
#endif

