/*
 *
 * Generic table driven parsing code to parse name=value pairs from a file.
 *   Copyright (c) International Business Machines  Corp., 2001
 *     IBM Contact: RCHNSSPT@us.ibm.com
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/********************************************************************
* Note: This code does not currently check for duplicate keywords in
*       the config file - the last one wins.
********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "prop_parse.h"


/* Local constants */
#define PARSE_WHITESPACE   " \t\n\r"
#define PARSE_COMMENT_CHARS "#;"
#define PARSE_EQUAL "="


/********************************************************************
*
* parseLine() - Parse a line into a keyword/value pair.  Commented
*               lines return special status and keyword/value are set
*               to NULL.
*
********************************************************************/
static int parseLine(char *line, char **returnKey, char **returnValue)
{
    int  retStatus;
    char *cur = line;
    char *key;
    char *keyEnd;
    char *value;

    /* Skip any leading whitespace */
    cur = line + strspn(cur, PARSE_WHITESPACE);

    /* Check for blank line or comment */
    if ((cur[0] == '\0') || (strchr(PARSE_COMMENT_CHARS, cur[0]) != NULL))
    {
        key = NULL;
        value = NULL;
        retStatus = PARSE_ERROR_COMMENT;
        goto EXIT;
    }

    /* First token should be the keyword */
    key = cur;

    /* Find end of keyword */
    cur = cur + strcspn(cur, PARSE_WHITESPACE PARSE_EQUAL);
    keyEnd = cur; /* Remember end of keyword */

    /* Skip over whitespace if any */
    cur = cur + strspn(cur, PARSE_WHITESPACE);
    /* Next char should be equal sign */
    if (cur[0] != '=')
    {
        key = NULL;
        value = NULL;
        retStatus = PARSE_ERROR_SYNTAX;
        goto EXIT;
    }

    /* Skip over equal sign */
    cur++;
    /* Its safe to terminate key string now */
    keyEnd[0] = '\0';
    /*printf("key: len=%d str=%s\n", strlen(key), key); */

    /* Skip whitespace after equal sign */
    cur = cur + strspn(cur, PARSE_WHITESPACE);

    /* Remainder of line is value - may be empty. */
    value = cur;
    if(value[0] != '\0')
    {
        char *endValue;
        /* Strip trailing whitespace */
        endValue = value + strlen(value)-1;
        while(strchr(PARSE_WHITESPACE, *endValue) != NULL)
            endValue--;
        
        /* Terminate value string at start of white space */
        endValue[1] = '\0';
    }

    /*printf("value: len=%d str=%s\n", strlen(value), value); */

    retStatus = PARSE_ERROR_SUCCESS;        

EXIT:
    *returnKey = key;
    *returnValue = value;
    return(retStatus);
}

/********************************************************************
*
* readParms() - Reads a config file and updates the parmTable with
* specified values.
*
********************************************************************/
int readParms(const char *configFilePath, lookupEntry parmTable[])
{
    int     retStatus = PARSE_ERROR_UNKNOWN;
    FILE    *f = NULL;
    char    lineBuf[512];
    int     lineCount=0;
    char    *curLine;
    char    *key;
    char    *value;

    /* Open config file */
    f = fopen(configFilePath, "r");
    if (f == NULL)
    {
        retStatus = PARSE_ERROR_OPEN_FILE;
        goto EXIT;
    }

    /* Process each line in config file */
    while ((curLine = fgets(lineBuf, sizeof(lineBuf), f)) != NULL)
    {
        int parseStatus;
        int keyIndex;

        lineCount++;
        parseStatus = parseLine(curLine, &key, &value);

        /* Skip comments */
        if(parseStatus == PARSE_ERROR_COMMENT)
            continue;

        /* Check for parsing error */
        if(parseStatus != PARSE_ERROR_SUCCESS)
        {
            /*?? Cannot log error because we do not have pam handle */
            /* printf("Parsing error=%d at line %d\n", parseStatus, lineCount); */
            continue;
        }

        /* Look up keyword */
        keyIndex=0;
        while (parmTable[keyIndex].keyword != NULL)
        {
            /* Look for keyword match in table */
            if (strcmp(key, parmTable[keyIndex].keyword) == 0)
            {
                /* Found a match, override default if value is not empty string */
                if (value[0] != '\0')
                {
                    /* if(strlen(value) > (sizeof(parmTable[keyIndex].value)-1))
                        printf("Warning: value for keyword=%s has been truncated\n", parmTable[keyIndex].keyword);
                    */
                    /* Copy value into table and override default */
                    strncpy(parmTable[keyIndex].value, value, sizeof(parmTable[keyIndex].value));
                    parmTable[keyIndex].value[sizeof(parmTable[keyIndex].value)-1] = '\0';
                }
                break;
            }
            keyIndex++;
        }

        /* Report error if keyword was not found in table */
        /*
        if ( parmTable[keyIndex].keyword == NULL )
        {
            printf("Keyword \"%s\" is not valid\n", key);
        } */

        /* Get another line */
    }

    retStatus = PARSE_ERROR_SUCCESS;

EXIT:
    /* Close file if open */
    if(f != NULL)
        fclose(f);

    return(retStatus);
}                                


