/*
 * Copyright information at end of file.
 */

#define _BSD_SOURCE

#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <string.h>
#include <malloc.h>
#include <pwd.h>
#include <shadow.h>
#include <limits.h>
#include <utmp.h>

#include <security/_pam_macros.h>
#include <security/pam_modules.h>

#include "support.h"

/* syslogging function for errors and other information */

void _log_err(int err, pam_handle_t *pamh, const char *format,...)
{
    char *service = NULL;
    char logname[256];
    va_list args;

    pam_get_item(pamh, PAM_SERVICE, (const void **) &service);
    if (service) {
        strncpy(logname, service, sizeof(logname));
        logname[sizeof(logname) - 1 - strlen("(pam_cifs)")] = '\0';
        strncat(logname, "(pam_cifs)", strlen("(pam_cifs)"));
    } else {
        strncpy(logname, "pam_cifs", sizeof(logname) - 1);
    }

    va_start(args, format);
    openlog(logname, LOG_CONS | LOG_PID, LOG_AUTH);
    vsyslog(err, format, args);
    va_end(args);
    closelog();
}

/* this is a front-end for module-application conversations */

static int converse(pam_handle_t * pamh, int ctrl, int nargs
            ,struct pam_message **message
            ,struct pam_response **response)
{
    int retval;
    struct pam_conv *conv;

    D(("begin to converse"));

    retval = pam_get_item(pamh, PAM_CONV, (const void **) &conv);
    if (retval == PAM_SUCCESS) {

        retval = conv->conv(nargs, (const struct pam_message **) message
                    ,response, conv->appdata_ptr);

        D(("returned from application's conversation function"));

        if (retval != PAM_SUCCESS && on(CIFS_DEBUG, ctrl)) {
            _log_err(LOG_DEBUG, pamh, "conversation failure [%s]"
                 ,pam_strerror(pamh, retval));
        }
    } else if (retval != PAM_CONV_AGAIN) {
        _log_err(LOG_ERR, pamh
                 ,"couldn't obtain coversation function [%s]"
             ,pam_strerror(pamh, retval));
    }
    D(("ready to return from module conversation"));

    return retval;      /* propagate error status */
}

int _make_remark(pam_handle_t * pamh, unsigned int ctrl
               ,int type, const char *text)
{
    int retval = PAM_SUCCESS;

    if (off(CIFS__QUIET, ctrl)) {
        struct pam_message *pmsg[1], msg[1];
        struct pam_response *resp;

        pmsg[0] = &msg[0];
        msg[0].msg = text;
        msg[0].msg_style = type;

        resp = NULL;
        retval = converse(pamh, ctrl, 1, pmsg, &resp);

        if (resp) {
            _pam_drop_reply(resp, 1);
        }
    }
    return retval;
}

  /*
   * Because getlogin() is broken and sometimes it just
   * doesn't work, we reimplement it here.
   */
char *PAM_getlogin(void)
{
    struct utmp *ut, line;
    char *curr_tty, *retval;
    static char curr_user[sizeof(ut->ut_user) + 4];

    retval = NULL;

    curr_tty = ttyname(0);
    if (curr_tty != NULL) {
        D(("PAM_getlogin ttyname: %s", curr_tty));
        curr_tty += 5;
        setutent();
        strncpy(line.ut_line, curr_tty, sizeof line.ut_line);
        if ((ut = getutline(&line)) != NULL) {
            strncpy(curr_user, ut->ut_user, sizeof(ut->ut_user));
            retval = curr_user;
        }
        endutent();
    }
    D(("PAM_getlogin retval: %s", retval));

    return retval;
}

/*
 * set the control flags for the CIFS module.
 */

int _set_ctrl(pam_handle_t *pamh, int flags, int *remember, int argc,
              const char **argv)
{
    unsigned int ctrl;

    D(("called."));

    ctrl = CIFS_DEFAULTS;   /* the default selection of options */

    /* set some flags manually */

    if (getuid() == 0 && !(flags & PAM_CHANGE_EXPIRED_AUTHTOK)) {
        D(("IAMROOT"));
        set(CIFS__IAMROOT, ctrl);
    }
    if (flags & PAM_UPDATE_AUTHTOK) {
        D(("UPDATE_AUTHTOK"));
        set(CIFS__UPDATE, ctrl);
    }
    if (flags & PAM_PRELIM_CHECK) {
        D(("PRELIM_CHECK"));
        set(CIFS__PRELIM, ctrl);
    }
    if (flags & PAM_DISALLOW_NULL_AUTHTOK) {
        D(("DISALLOW_NULL_AUTHTOK"));
        set(CIFS__NONULL, ctrl);
    }
    if (flags & PAM_SILENT) {
        D(("SILENT"));
        set(CIFS__QUIET, ctrl);
    }
    /* now parse the arguments to this module */

    while (argc-- > 0) {
        int j;

        D(("pam_cifs arg: %s", *argv));

        for (j = 0; j < CIFS_CTRLS_; ++j) {
            if (cifs_args[j].token
                && !strncmp(*argv, cifs_args[j].token, strlen(cifs_args[j].token))) {
                break;
            }
        }

        if (j >= CIFS_CTRLS_) {
            _log_err(LOG_ERR, pamh,
                     "unrecognized option [%s]", *argv);
        } else {
            ctrl &= cifs_args[j].mask;  /* for turning things off */
            ctrl |= cifs_args[j].flag;  /* for turning things on  */

            if (remember != NULL) {
                if (j == CIFS_REMEMBER_PASSWD) {
                    *remember = strtol(*argv + 9, NULL, 10);
                    if ((*remember == LONG_MIN) || (*remember == LONG_MAX))
                        *remember = -1;
                    if (*remember > 400)
                        *remember = 400;
                }
            }
        }

        ++argv;     /* step to next argument */
    }

    /* auditing is a more sensitive version of debug */

    if (on(CIFS_AUDIT, ctrl)) {
        set(CIFS_DEBUG, ctrl);
    }
    /* return the set of flags */

    D(("done."));
    return ctrl;
}

static void _cleanup(pam_handle_t * pamh, void *x, int error_status)
{
    _pam_delete(x);
}

/* ************************************************************** *
 * Useful non-trivial functions                                   *
 * ************************************************************** */

  /*
   * the following is used to keep track of the number of times a user fails
   * to authenticate themself.
   */

#define FAIL_PREFIX                   "-CIFS-FAIL-"
#define CIFS_MAX_RETRIES              3

struct _pam_failed_auth {
    char *user;     /* user that's failed to be authenticated */
    char *name;     /* attempt from user with name */
    int uid;        /* uid of calling user */
    int euid;       /* euid of calling process */
    int count;      /* number of failures so far */
};

#ifndef PAM_DATA_REPLACE
#error "Need to get an updated libpam 0.52 or better"
#endif

/*
 * obtain a password from the user
 */

int _cifs_read_password(pam_handle_t * pamh
            ,unsigned int ctrl
            ,const char *comment
            ,const char *prompt1
            ,const char *prompt2
            ,const char *data_name
            ,const char **pass)
{
    int authtok_flag;
    int retval;
    const char *item;
    char *token;

    D(("called"));

    /*
     * make sure nothing inappropriate gets returned
     */

    *pass = token = NULL;

    /*
     * which authentication token are we getting?
     */

    authtok_flag = on(CIFS__OLD_PASSWD, ctrl) ? PAM_OLDAUTHTOK : PAM_AUTHTOK;

    /*
     * should we obtain the password from a PAM item ?
     */

    if (on(CIFS_TRY_FIRST_PASS, ctrl) || on(CIFS_USE_FIRST_PASS, ctrl)) {
        retval = pam_get_item(pamh, authtok_flag, (const void **) &item);
        if (retval != PAM_SUCCESS) {
            /* very strange. */
            _log_err(LOG_ALERT, pamh
                 ,"pam_get_item returned error to cifs-read-password"
                );
            return retval;
        } else if (item != NULL) {  /* we have a password! */
            *pass = item;
            item = NULL;
            return PAM_SUCCESS;
        } else if (on(CIFS_USE_FIRST_PASS, ctrl)) {
            return PAM_AUTHTOK_RECOVER_ERR;     /* didn't work */
        } else if (on(CIFS_USE_AUTHTOK, ctrl)
               && off(CIFS__OLD_PASSWD, ctrl)) {
            return PAM_AUTHTOK_RECOVER_ERR;
        }
    }
    /*
     * getting here implies we will have to get the password from the
     * user directly.
     */

    {
        struct pam_message msg[3], *pmsg[3];
        struct pam_response *resp;
        int i, replies;

        /* prepare to converse */

        if (comment != NULL && off(CIFS__QUIET, ctrl)) {
            pmsg[0] = &msg[0];
            msg[0].msg_style = PAM_TEXT_INFO;
            msg[0].msg = comment;
            i = 1;
        } else {
            i = 0;
        }

        pmsg[i] = &msg[i];
        msg[i].msg_style = PAM_PROMPT_ECHO_OFF;
        msg[i++].msg = prompt1;
        replies = 1;

        if (prompt2 != NULL) {
            pmsg[i] = &msg[i];
            msg[i].msg_style = PAM_PROMPT_ECHO_OFF;
            msg[i++].msg = prompt2;
            ++replies;
        }
        /* so call the conversation expecting i responses */
        resp = NULL;
        retval = converse(pamh, ctrl, i, pmsg, &resp);

        if (resp != NULL) {

            /* interpret the response */

            if (retval == PAM_SUCCESS) {    /* a good conversation */

                token = x_strdup(resp[i - replies].resp);
                if (token != NULL) {
                    if (replies == 2) {

                        /* verify that password entered correctly */
                        if (!resp[i - 1].resp
                            || strcmp(token, resp[i - 1].resp)) {
                            _pam_delete(token); /* mistyped */
                            retval = PAM_AUTHTOK_RECOVER_ERR;
                            _make_remark(pamh, ctrl
                                    ,PAM_ERROR_MSG, MISTYPED_PASS);
                        }
                    }
                } else {
                    _log_err(LOG_NOTICE, pamh
                         ,"could not recover authentication token");
                }

            }
            /*
             * tidy up the conversation (resp_retcode) is ignored
             * -- what is it for anyway? AGM
             */

            _pam_drop_reply(resp, i);

        } else {
            retval = (retval == PAM_SUCCESS)
                ? PAM_AUTHTOK_RECOVER_ERR : retval;
        }
    }

    if (retval != PAM_SUCCESS) {
        if (on(CIFS_DEBUG, ctrl))
            _log_err(LOG_DEBUG, pamh,
                     "unable to obtain a password");
        return retval;
    }
    /* 'token' is the entered password */

    if (off(CIFS_NOT_SET_PASS, ctrl)) {

        /* we store this password as an item */

        retval = pam_set_item(pamh, authtok_flag, token);
        _pam_delete(token); /* clean it up */
        if (retval != PAM_SUCCESS
            || (retval = pam_get_item(pamh, authtok_flag
                          ,(const void **) &item))
            != PAM_SUCCESS) {

            _log_err(LOG_CRIT, pamh, "error manipulating password");
            return retval;

        }
    } else {
        /*
         * then store it as data specific to this module. pam_end()
         * will arrange to clean it up.
         */

        retval = pam_set_data(pamh, data_name, (void *) token, _cleanup);
        if (retval != PAM_SUCCESS) {
            _log_err(LOG_CRIT, pamh
                     ,"error manipulating password data [%s]"
                 ,pam_strerror(pamh, retval));
            _pam_delete(token);
            return retval;
        }
        item = token;
        token = NULL;   /* break link to password */
    }

    *pass = item;
    item = NULL;        /* break link to password */

    return PAM_SUCCESS;
}

/*
 *   Copyright (c) Jan Rkorajski 1999.
 *   Copyright (c) Andrew G. Morgan 1996-8.
 *   Copyright (c) Alex O. Yuriev, 1996.
 *   Copyright (c) Cristian Gafton 1996.
 *   Copyright (c) International Business Machines  Corp., 2001
 *     IBM Contact: RCHNSSPT@us.ibm.com
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
