/*
 * Copyright information at end of file.
 */

#ifndef _PAM_CIFS_SUPPORT_H
#define _PAM_CIFS_SUPPORT_H


/*
 * here is the string to inform the user that the new passwords they
 * typed were not the same.
 */

#define MISTYPED_PASS "Sorry, passwords do not match."

/* type definition for the control options */

typedef struct {
    const char *token;
    unsigned int mask;  /* shall assume 32 bits of flags */
    unsigned int flag;
} CIFS_Ctrls;

/*
 * macro to determine if a given flag is on
 */

#define on(x,ctrl)  (cifs_args[x].flag & ctrl)

/*
 * macro to determine that a given flag is NOT on
 */

#define off(x,ctrl) (!on(x,ctrl))

/*
 * macro to turn on/off a ctrl flag manually
 */

#define set(x,ctrl)   (ctrl = ((ctrl)&cifs_args[x].mask)|cifs_args[x].flag)
#define unset(x,ctrl) (ctrl &= ~(cifs_args[x].flag))

/* the generic mask */

#define _ALL_ON_  (~0U)

/* end of macro definitions for the control flags */

/* ****************************************************************** *
 * ctrl flags proper..
 */

/*
 * here are the various options recognized by the cifs module. They
 * are enumerated here and then defined below. Internal arguments are
 * given NULL tokens.
 */

#define CIFS__OLD_PASSWD          0 /* internal */
#define CIFS__VERIFY_PASSWD       1 /* internal */
#define CIFS__IAMROOT             2 /* internal */

#define CIFS_AUDIT                3 /* print more things than debug..
                       some information may be sensitive */
#define CIFS_USE_FIRST_PASS       4
#define CIFS_TRY_FIRST_PASS       5
#define CIFS_NOT_SET_PASS         6 /* don't set the AUTHTOK items */

#define CIFS__PRELIM              7 /* internal */
#define CIFS__UPDATE              8 /* internal */
#define CIFS__NONULL              9 /* internal */
#define CIFS__QUIET              10 /* internal */
#define CIFS_USE_AUTHTOK         11 /* insist on reading PAM_AUTHTOK */
#define CIFS_SHADOW              12 /* signal shadow on */
#define CIFS_MD5_PASS            13 /* force the use of MD5 passwords */
#define CIFS__NULLOK             14 /* Null token ok */
#define CIFS_DEBUG               15 /* send more info to syslog(3) */
#define CIFS_NODELAY             16 /* admin does not want a fail-delay */
#define CIFS_NIS                 17 /* wish to use NIS for pwd */
#define CIFS_BIGCRYPT            18 /* use DEC-C2 crypt()^x function */
#define CIFS_LIKE_AUTH           19 /* need to auth for setcred to work */
#define CIFS_REMEMBER_PASSWD     20 /* Remember N previous passwords */
/* -------------- */
#define CIFS_CTRLS_              21 /* number of ctrl arguments defined */


/* erbjor??? Remove options that are not relevant to NT? */
static const CIFS_Ctrls cifs_args[CIFS_CTRLS_] =
{
/* symbol                  token name          ctrl mask             ctrl     *
 * ----------------------- ------------------- --------------------- -------- */

/* CIFS__OLD_PASSWD */     {NULL,              _ALL_ON_,                  01},
/* CIFS__VERIFY_PASSWD */  {NULL,              _ALL_ON_,                  02},
/* CIFS__IAMROOT */        {NULL,              _ALL_ON_,                  04},
/* CIFS_AUDIT */           {"audit",           _ALL_ON_,                 010},
/* CIFS_USE_FIRST_PASS */  {"use_first_pass",  _ALL_ON_^(060),           020},
/* CIFS_TRY_FIRST_PASS */  {"try_first_pass",  _ALL_ON_^(060),           040},
/* CIFS_NOT_SET_PASS */    {"not_set_pass",    _ALL_ON_,                0100},
/* CIFS__PRELIM */         {NULL,              _ALL_ON_^(0600),         0200},
/* CIFS__UPDATE */         {NULL,              _ALL_ON_^(0600),         0400},
/* CIFS__NONULL */         {NULL,              _ALL_ON_,               01000},
/* CIFS__QUIET */          {NULL,              _ALL_ON_,               02000},
/* CIFS_USE_AUTHTOK */     {"use_authtok",     _ALL_ON_,               04000},
/* CIFS_SHADOW */          {"shadow",          _ALL_ON_,              010000},
/* CIFS_MD5_PASS */        {"md5",             _ALL_ON_^(0400000),    020000},
/* CIFS__NULLOK */         {"nullok",          _ALL_ON_^(01000),           0},
/* CIFS_DEBUG */           {"debug",           _ALL_ON_,              040000},
/* CIFS_NODELAY */         {"nodelay",         _ALL_ON_,             0100000},
/* CIFS_NIS */             {"nis",             _ALL_ON_^(010000),    0200000},
/* CIFS_BIGCRYPT */        {"bigcrypt",        _ALL_ON_^(020000),    0400000},
/* CIFS_LIKE_AUTH */       {"likeauth",        _ALL_ON_,            01000000},
/* CIFS_REMEMBER_PASSWD */ {"remember=",       _ALL_ON_,            02000000},
};

/* erbjor??? Not honoring CIFS_NONULL, so remove? */
#define CIFS_DEFAULTS  (cifs_args[CIFS__NONULL].flag)


/* use this to free strings. ESPECIALLY password strings */

#define _pam_delete(xx)     \
{               \
    _pam_overwrite(xx); \
    _pam_drop(xx);      \
}

extern char *PAM_getlogin(void);
extern void _log_err(int err, pam_handle_t *pamh, const char *format,...);
extern int _make_remark(pam_handle_t * pamh, unsigned int ctrl
               ,int type, const char *text);
extern int _set_ctrl(pam_handle_t * pamh, int flags, int *remember, int argc,
             const char **argv);
extern int _cifs_read_password(pam_handle_t * pamh
            ,unsigned int ctrl
            ,const char *comment
            ,const char *prompt1
            ,const char *prompt2
            ,const char *data_name
            ,const char **pass);

#endif /* _PAM_CIFS_SUPPORT_H */

/*
 *   Copyright (c) Jan Rkorajski 1999.
 *   Copyright (c) Andrew G. Morgan 1996-8.
 *   Copyright (c) Alex O. Yuriev, 1996.
 *   Copyright (c) Cristian Gafton 1996.
 *   Copyright (c) International Business Machines  Corp., 2001
 *     IBM Contact: RCHNSSPT@us.ibm.com
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
