#!/bin/bash

# baseOSUpdate.sh
#
# Updates the baseOS files on the 16M flash card.
#
# The update is done in the server-booted case if
# the baseOS code on the server is from a later product install
# than the baseOS code on the 16M flash.  This prevents compatibility
# problems between applications and the baseOS on the 16M flash.
# The applications may depend on an updated version of a library.
#
# The update is done if the build date on the server is later than
# the build date on the client, *and* the CRC of the server files
# are different than the client files.  This prevents unneccessary
# updates to the 16M flash.
#
# In the flash-booted case a warning is displayed since we do
# not have a server to update from.

# TODO
# How to display pop up if flash booted
# error handing .... move old images and imageMounts to tmp in case it fails ????

# Set up base dirs on server and client - defaults to N70 server boot autoupdate settings    
SERVER_MOUNT=/mnt/serverhost
CLIENT_MOUNT=/
FORCE=0
LATER_BUILD_DATE=0
REBOOT=y
SOFTWARE_UPDATE=0

while getopts ufs:c:r: options
do
        case $options in
                s) SERVER_MOUNT=$OPTARG;;
		c) CLIENT_MOUNT=$OPTARG;;
                u) SOFTWARE_UPDATE=1;;
                f) FORCE=1;;
		r) REBOOT=$OPTARG     
        esac
done

mkdir -p /var/log/lcma
LOG_FILE=/var/log/lcma/baseOSUpdate.log

# Set needed env vars (MODEL_TYPE and BOOTSOURCE)
#. /$SERVER_MOUNT/etc/rc.d/rc.IBM_NS.env  cec

MSG="Checking for required update of boot partition\n"
echo -e $MSG | tee $LOG_FILE

if [ "$MODEL_TYPE" = "2200" -o "$MODEL_TYPE" = "2800" ]; then
        CLIENT_VERSION_FILE=${CLIENT_MOUNT}/IBM/version
        CLIENT_CRC_FILE=${CLIENT_MOUNT}/image/base/crc
        CLIENT_SDF_FILE=${CLIENT_MOUNT}/image/base/NetVista_BaseOS.sdf
else
        CLIENT_VERSION_FILE=${CLIENT_MOUNT}/local/version
        CLIENT_CRC_FILE=${CLIENT_MOUNT}/local/crc
        CLIENT_SDF_FILE=${CLIENT_MOUNT}/local/NetVista_BaseOS.sdf
fi

if [ $FORCE -eq 1 ]; then
        MSG="Force update has been selected\n"
        echo -e $MSG | tee $LOG_FILE
else
        # Get the build dates on the server and the client
        SERVER_BUILD_DATE=`awk -F[=] '/linux_dist_date/ {print $2}' $SERVER_MOUNT/IBM/version`
        SERVER_YEAR=`echo $SERVER_BUILD_DATE | cut -c5-6`
        SERVER_MONTH=`echo $SERVER_BUILD_DATE | cut -c1-2`
        SERVER_DAY=`echo $SERVER_BUILD_DATE | cut -c3-4`
        CLIENT_BUILD_DATE=`awk -F[=] '/linux_dist_date/ {print $2}' $CLIENT_VERSION_FILE`
        CLIENT_YEAR=`echo $CLIENT_BUILD_DATE | cut -c5-6`
        CLIENT_MONTH=`echo $CLIENT_BUILD_DATE | cut -c1-2`
        CLIENT_DAY=`echo $CLIENT_BUILD_DATE | cut -c3-4`

        # Check if the build date of the server is later than the build date on the client.
        if [ "$SERVER_YEAR" -gt "$CLIENT_YEAR" ]; then
                LATER_BUILD_DATE=1
        elif [ "$SERVER_YEAR" -eq "$CLIENT_YEAR" ] && [ "$SERVER_MONTH" -gt "$CLIENT_MONTH" ]; then
                LATER_BUILD_DATE=1
        elif [ "$SERVER_YEAR" -eq "$CLIENT_YEAR" ] && [ "$SERVER_MONTH" -eq "$CLIENT_MONTH" ] && [ "$SERVER_DAY" -gt "$CLIENT_DAY" ]; then
                LATER_BUILD_DATE=1
        fi
fi

if [ $FORCE -eq 1 -o $LATER_BUILD_DATE -eq 1 ]; then
        # Update is being forced, or the build on the server is later than the client

        # If force is off, then compare the CRCs in the server and client baseOS SDFs
        if [ $FORCE -eq 0 ]; then
                # Get the server cram SDF CRC
                SERVER_CRAM_SDF=$SERVER_MOUNT/IBM/update/sdf/NetVista_BaseOS.sdf
                SERVER_CRAM_CRC=`sed -n '1p' $SERVER_CRAM_SDF | awk -F'[ =]' '{\
                        for (i = 1; i <= NF; i++)\
                        {\
				tag = toupper($i);\
                                if (tag == "CRC")\
                                {\
                                        print $(i+1)\
                                }\
                        }\
                }'`

                # Get the client cram SDF CRC
                CLIENT_CRAM_CRC=x
                if [ -f $CLIENT_SDF_FILE ]; then 
                        CLIENT_CRAM_CRC=`sed -n '1p' $CLIENT_SDF_FILE | awk -F'[ =]' '{\
                                for (i = 1; i <= NF; i++)\
                                {\
                                        tag = toupper($i);\
                                        if (tag == "CRC")\
                                        {\
                                                print $(i+1)\
                                        }\
                                }\
                        }'`
                fi

                # Read in the CRCs for the server and client base tar file
                if [ "$MODEL_TYPE" = "2200" -o "$MODEL_TYPE" = "2800" ]; then
                        SERVER_TAR_CRC_FILE=$SERVER_MOUNT/IBM/update/base/crc.2x00
                else
                        SERVER_TAR_CRC_FILE=$SERVER_MOUNT/IBM/update/base/crc.n70
                fi

                . $SERVER_TAR_CRC_FILE  # Load SERVER_CRC for the server base tar crc
                . $CLIENT_CRC_FILE      # Load CRC for the client base tar crc     

                # If server and client CRCs are the same then no need to update.
                if [ "$SERVER_CRAM_CRC" = "$CLIENT_CRAM_CRC" ] && [ "$SERVER_CRC" = "$CRC" ]; then
                        MSG="No update needed.  Build dates are different, but CRCs are the same."
                        echo -e $MSG | tee -a $LOG_FILE
                        exit 0
                fi
        fi

        # Check if flash booted.
        if [ "$BOOTSOURCE" = "flash" -a $SOFTWARE_UPDATE -eq 0 ]; then
                # Flash booted and apps are later than base.
                # Don't have a server to update the 16M flash, so put up a warning and exit.
                echo "WARNING: Later service update on app flash"  #??????
                echo -e "Error: Unable to do update on flash-booted workstation. Exiting" >> $LOG_FILE
                exit 1
        fi

        MSG="Boot partition update is needed"
        echo -e $MSG | tee -a $LOG_FILE

        # Determine the files to copy, based on the model
        SERVERCRAMIMAGE=$SERVER_MOUNT/IBM/update/base/base.cramfs
        if [ "$MODEL_TYPE" = "2200" -o "$MODEL_TYPE" = "2800" ]; then
                SERVERTAR=$SERVER_MOUNT/IBM/update/base/base_2x00.tgz
        else
                SERVERTAR=$SERVER_MOUNT/IBM/update/base/base_n70.tgz
        fi

        if [ -f "$SERVERCRAMIMAGE" -a -f "$SERVERTAR" ]; then

                if [ $SOFTWARE_UPDATE -eq 1 ]; then
                        /sbin/mke2fs -q -m0 /dev/ram2 10240
                        if [ ! -d /tmp/base ]; then
                                mkdir /tmp/base
                        fi
                        /bin/mount -n /dev/ram2 /tmp/base
                        cp ${CLIENT_MOUNT}/base.cramfs /tmp/base
                        if [ "$MODEL_TYPE" = "N70" -o "$MODEL_TYPE" = "N50" -o "$MODEL_TYPE" = "BIOS" ]; then
                                mkdir -p /mnt/serverhost/mnt/base
                                mount -n -o loop=/dev/loop /tmp/base/base.cramfs /mnt/serverhost/mnt/base
                        else
                                mkdir -p /mnt/base
                                mount -n -o loop=/dev/loop /tmp/base/base.cramfs /mnt/base
                        fi
                        # Note - PATH and LD_LIBRARY_PATH were exported by the caller
                fi

                # Remount the first partition r/w on N70
		if [ "$MODEL_TYPE" = "N70" -o "$MODEL_TYPE" = "N50" -o "$MODEL_TYPE" = "BIOS" ]; then
                	mount -n -o remount,rw / / >> $LOG_FILE 2>&1
		fi

                # Copy the baseOS cramfs image to 16M flash.
                MSG="Copying compressed base operating system image to boot partition.."
                echo -e $MSG | tee -a $LOG_FILE
                cp -a $SERVERCRAMIMAGE ${CLIENT_MOUNT}/base.cramfs >> $LOG_FILE 2>&1

                # Copy the uncompressed files to the 16M flash
                MSG="Copying uncompressed files to boot partition.."
                echo -e $MSG | tee -a $LOG_FILE
                tar -xzf $SERVERTAR -C $CLIENT_MOUNT >> $LOG_FILE 2>&1
		sync; sync; sync
		if [ "$MODEL_TYPE" = "BIOS" ]; then
                    echo "Testing if tar command was succesful."
                    # sometimes the tar command doesnt work, lets check if it did
		    while [ `ls -l /mount | awk {'print $5'}` != "552628" -o `ls -l /local/NetVista_BaseOS.sdf | awk {'print $5'}` = "0" ]; do
                       # mount is not right size , tar must have failed
                       echo "The tar command failed!!! Trying again."
                       tar -xzf $SERVERTAR -C $CLIENT_MOUNT >> $LOG_FILE 2>&1
                       sleep 1
                       sync
                    done
                    echo "The tar command was succesful."
                    sleep 2
		fi

		# DO CHECKS FOR N70 vs BIOS clients:
                if [ "$MODEL_TYPE" = "BIOS" ]; then
                   # the base_n70.tgz file has been laid down to the first partition.
		   # make sure we undo the changes that break OEM support.
                   # inittab, rc.OEM scripts and /boot need to be replaced. 
		   # files will be in $SERVER_MOUNT/IBM/update/base/OEM
                   rm -Rf /boot
                   rm -f /etc/inittab
                   rm -f /etc/bootstrap.d/rc.sysinit.OEM_NS.bringup
                   cp -f $SERVER_MOUNT/IBM/update/base/OEM/inittab.OEM $CLIENT_MOUNT/etc/inittab
                   cp -f $SERVER_MOUNT/IBM/update/base/OEM/rc.sysinit.OEM_NS.bringup $CLIENT_MOUNT/etc/bootstrap.d/rc.sysinit.OEM_NS.bringup
                   chmod 755 $CLIENT_MOUNT/etc/bootstrap.d/rc.sysinit.OEM_NS.bringup
                   cp -Rf $SERVER_MOUNT/IBM/update/base/OEM/boot.OEM $CLIENT_MOUNT/boot
                fi

                # Make sure the files have been saved.
		sync ; sync ; sync

		if [ "$MODEL_TYPE" != "BIOS" ]; then
			if [ "$MODEL_TYPE" = "N70" -o "$MODEL_TYPE" = "N50" ]; then
                        	# Set up the boot record on N70
                        	MSG="Creating boot sector.."
                        	echo -e $MSG | tee -a $LOG_FILE
                        	lilo -v -v -C ${CLIENT_MOUNT}/configs/lilo.conf >> $LOG_FILE 2>&1
                	else
                        	# Setup dev directory.
                        	mkdir ${CLIENT_MOUNT}/dev

                        	if [ ! -e ${CLIENT_MOUNT}/kernel.2200 ]; then
                                	ln ${CLIENT_MOUNT}/kernel.2x00 ${CLIENT_MOUNT}/kernel.2200 
                        	fi
                        	if [ ! -e ${CLIENT_MOUNT}/kernel.2800 ]; then
                        	        ln ${CLIENT_MOUNT}/kernel.2x00 ${CLIENT_MOUNT}/kernel.2800 
				fi
                	fi
		fi

                # Remount the first partition r/o on N70 to sync changes.
		if [ "$MODEL_TYPE" = "N70" -o "$MODEL_TYPE" = "N50" -o "$MODEL_TYPE" = "BIOS" ]; then
                	mount -n -o remount,ro / / >> $LOG_FILE 2>&1
                        mount --bind $SERVER_MOUNT/etc /etc >> $LOG_FILE 2>&1
		fi

		MSG="Done updating the boot partition"
		echo -e $MSG | tee -a $LOG_FILE

                # Reboot, if requested
		if [ "$REBOOT" = "y" ]; then
                	MSG="Rebooting..."       
               		echo -e $MSG | tee -a $LOG_FILE
			if [ $SOFTWARE_UPDATE -eq 1 ]; then
				/IBM/bin/writeNvram 78 1
				sync
			fi 

                        if [ "$MODEL_TYPE" = "N70" -o "$MODEL_TYPE" = "N50" -o "$MODEL_TYPE" = "BIOS" ]; then
				shutdown -n -r now
			else
	               		/IBM/bin/reboot_IBM_NS 5 >> $LOG_FILE 2>&1
			fi
		fi
        else
                MSG="Error: Unable to find cramfs image and tar on the server: $SERVERCRAMIMAGE $SERVERTAR"
                echo -e $MSG | tee -a $LOG_FILE
       
        fi
else
        MSG="No update needed.  Build dates are the same"
        echo -e $MSG | tee -a $LOG_FILE
fi
