#!/bin/sh

##
# Create the compressed portion of the Image Description File (IDF) for the specified tgz file
# in the /tmp directory to be evaluated by the IDFBuilder java application for an accurate 
# flash-card ext2 filesystem disk usage measurement.
#
# @param $1 (required) base directory to find the /IBM/update/idf directory. e.g. /nstation/linux, /
# @param $2 (required) IDF path and file name relative to the base directory. e.g. /IBM/update/idf/NetVista_2x00_32M.idf
# @param $3 (required) name of *.tgz file for which to search in all of the IDF's SDFs. e.g. usr.share.tgz
# @param $4 (optional) "-v" which turns on verbose messaging. 
#
# @returns string fully-qualified path and file name of the cram-compressed file for the IDFBuilder java 
#  get correct disk usage measurements.  If no file name is returned (i.e. an empty string), then there 
#  is no compressed data for the specified tgz file.
##

boot_dir=$1
idf=$boot_dir/$2
tgz=$3

if [ "$4" = "-v" ] ; then
  verbose="true"
fi

flash_block_size=1024

tmp_dir=/tmp/__`basename $idf`__
compressed_base_dir=$tmp_dir/compressed
cram_dir=$compressed_base_dir/$tgz
cram_file=$compressed_base_dir/"$tgz".cramfs


if [ -n "$verbose" ] ; then
  echo boot_dir=$boot_dir
  echo idf=$idf
  echo cram_dir=$cram_dir
  echo cram_file=$cram_file
fi

mkdir -p $cram_dir
cd $cram_dir

let exist_count=0

for sdf in `cat $idf | sed -e "s/SDF.*SRC=//" -e "/^HEAD/d" | cut -d' ' -f1`; do
     
  sdfdir=`echo $sdf | sed -e "s/\.sdf$//"`
  sdf_tar=$boot_dir/$sdfdir/$tgz

  if [ -n "$verbose" ] ; then
    echo sdf_tar=$sdf_tar
  fi

  if [ -e $sdf_tar ] ; then 
    if [ -n "$verbose" ] ; then
      echo "  " $sdf_tar EXISTS 
    fi
    let exist_count=exist_count+1
  else
    if [ -n "$verbose" ] ; then
      echo "  " $sdf_tar does not exist.
    fi
  fi

  # Note that files could be overwritten.  No way to currently account for that.
  tar xzf $sdf_tar >/dev/null 2>&1
done

if [ -n "$verbose" ] ; then
  echo exist_count=$exist_count
fi

# If we've found the specified tgz file in any of the SDFs, then
# we create a cram file and return the cram file's name.

if [ $exist_count -gt 0 ] ; then
  /sbin/mkcramfs $cram_dir $cram_file > /dev/null
  echo $cram_file

# Else we return an empty string indicating the specified tgz file
# was not found for any of the SDFs.

else
  echo ""
fi

rm -rf $cram_dir

