#!/bin/sh

if [ -f /IBM/bin/profile.functions ]; then
   . /IBM/bin/profile.functions
fi


debug_echo() {

   if [ "$DEBUG" = "true" ]; then
      echo $* >&2
   fi
}


display_help () {

   echo
   echo "Syntax: `basename $0` [options...] <filename> <url> [<proxy_server>[:<proxy_port>]]"
   echo "<filename>     Profile zip file (generated from create_profile.sh). No path, just the filename."
   echo "<url>          URL to export the profile. Specify boot location as boot://"
   echo "<proxy_server> IP address of the proxy server."
   echo "<proxy_port>   Port number of the proxy server."
   echo "Options:"
   echo " -b/--base <dir>       Profile base directory"
   echo " -f/--file <filename>  File to save as"    
   echo " -h/-?/--help          This help text"
   echo " -m/--manifest <file>  Use alternate manifest file"
   echo

   exit 1
}

parse_options () {

   HELP=false
   DEBUG=false
   BASE=/tmp/profile/generated
   FILENAME=
   MANIFEST_FILE=/tmp/profile/machine/manifest.properties
   PROFILE_TYPE=
   PROFILE_FILE=
   PROXY=
   URL=

   while [ -n "$1" ]; do

      case $1 in
         "-b"|"--base"):
            shift
            if [ -n "$1" ]; then
               BASE=$1
            else
               HELP=true
            fi
            ;;
         "-f"|"--file"):
            shift
            if [ -n "$1" ]; then
               # Force the filename to uppercase to handle Windows NT $?!@*!-edness.
               FILENAME=`echo $1 | tr '[a-z]' '[A-Z]'`
            else
               HELP=true
            fi
            ;;
         "--debug"):
            DEBUG=true
            ;;
         "-h"|"-?"|"--help"):
            HELP=true
            ;;
         "-m"|"--manifest"):
            shift
            if [ -n "$1" ]; then
               MANIFEST_FILE=$1
            else
               HELP=true
            fi
            ;;
         *):
            PROFILE_FILE=$1

            __profile_comment=`/usr/bin/unzip -Z -z $BASE/$ComponentDirectory/$PROFILE_FILE -x`
            if [ -z "$__profile_comment" ]; then
               echo "Error. Bad zip file: $BASE/$ComponentDirectory/$PROFILE_FILE"
               HELP=true
            fi

            if [ -n "$__profile_comment" ]; then
               eval export $__profile_comment
            fi
            __key_list=`echo $__profile_comment | sed -e s/[\\\]\"//g -e s/=\"[^\"]*\"\ /\ /g | cut -d= -f1`

            PROFILE_TYPE=$ComponentType

            unset __profile_comment
            unset __key_list

            shift
            if [ -n "$1" ]; then
               URL=$1
 
               shift
               if [ -n "$1" ]; then
                  PROXY=$1
               fi
            else
               HELP=true
            fi

            break               
            ;;
      esac

      shift
   done

   if [ -z "$PROFILE_FILE" ]; then
      echo "No profile file specified"
      HELP=true
   fi

   if [ -z "$URL" ]; then
      echo "No url specified"
      HELP=true
   fi

   debug_echo "BASE=$BASE"
   debug_echo "FILENAME=$FILENAME"
   debug_echo "MANIFEST_FILE=$MANIFEST_FILE"
   debug_echo "PROFILE_TYPE=$PROFILE_TYPE"
   debug_echo "PROFILE_FILE=$PROFILE_FILE"
   debug_echo "URL=$URL"
   debug_echo "PROXY=$PROXY"
   debug_echo

   if [ "$HELP" = "true" ]; then
      display_help
   fi

}



generate_output_filename () {

   case "$OutputFormat" in

      mac):
         __output_file="$MACADDRESS.ZIP"
         ;;
      ip):
         __output_file="$ip.ZIP"
         ;;
      other|*):
         __output_file="${FILENAME:-DEFAULT}.ZIP"
         ;;
   esac

   # Force to uppercase.
   __output_file="`echo $__output_file | tr '[a-z]' '[A-Z]'`"

   echo $__output_file
   unset __output_file
}

upload_profile () {

   OUTPUT_FILE=`generate_output_filename`

   __protocol=`echo $URL | cut -d: -f1`
   __server=`echo $URL | sed s/^.*:\\\/\\\/// | cut -d/ -f1`
   __path=`echo $URL | sed s/^.*:\\\/\\\/[^\\/]*//`

   debug_echo "$__protocol   $__server   $__path"

   case $__protocol in

      file):
         debug_echo "cp -f $BASE/$PROFILE_FILE $__path/$ComponentDirectory/$OUTPUT_FILE"
         mkdir -p $__path/$ComponentDirectory
         cp -f $BASE/$PROFILE_FILE $__path/$ComponentDirectory/$OUTPUT_FILE
         if [ "$?" = "0" ]; then
            return 0
         fi
         ;;

      nfs):
         conditional_umount /tmp/profile/mnt/profiles
         mkdir -p /tmp/profile/mnt/profiles
         if conditional_mount $URL /tmp/profile/mnt/profiles; then
            debug_echo "cp -f $BASE/$PROFILE_FILE /tmp/profile/mnt/profiles/$ComponentDirectory/$OUTPUT_FILE"
            mkdir -p /tmp/profile/mnt/profiles/$ComponentDirectory
            cp -f $BASE/$PROFILE_FILE /tmp/profile/mnt/profiles/$ComponentDirectory/$OUTPUT_FILE
            rc=$?
            umount /tmp/profile/mnt/profiles
            if [ "$rc" = "0" ]; then
               return 0
            fi
         fi
         ;;

      http|https):
         debug_echo "curl \"$URL/GetProfiles?TCMLocation=\" -s"
         __tcm_response=`curl "$URL/GetProfiles?TCMLocation=" -s`
         if [ -n "$__tcm_response" ]; then
            eval export $__tcm_response
            unset __tcm_response

            if [ -n "$TCMPROFILES" ]; then
               debug_echo "curl -T $BASE/$PROFILE_FILE \"$TCMPROFILES/$ComponentDirectory/$OUTPUT_FILE\" -s"
               curl -T $BASE/$PROFILE_FILE "$TCMPROFILES/$ComponentDirectory/$OUTPUT_FILE" -s
               rc=$?

               # Inform TCM that we saved a profile.
               debug_echo "curl \"$URL/GetProfiles?TCMChangedProfileComponent=$ComponentDirectory/$OUTPUT_FILE\" -s > /dev/null"
               curl "$URL/GetProfiles?TCMChangedProfileComponent=$ComponentDirectory/$OUTPUT_FILE" -s > /dev/null

               if [ "$rc" = "0" ]; then
                  return 0
               fi
            fi
         fi
         ;;

      boot):
         conditional_umount /tmp/profile/mnt/profiles
         mkdir -p /tmp/profile/mnt/profiles
         if [ "$BOOTSOURCE" != "flash" ]; then
            URL="nfs://$server`dirname $serverBaseDir`/profiles"
            debug_echo "Export to boot location $URL"
            if conditional_mount $URL /tmp/profile/mnt/profiles; then
               debug_echo "cp -f $BASE/$PROFILE_FILE /tmp/profile/mnt/profiles/$ComponentDirectory/$OUTPUT_FILE"
               mkdir -p /tmp/profile/mnt/profiles/$ComponentDirectory
               cp -f $BASE/$PROFILE_FILE /tmp/profile/mnt/profiles/$ComponentDirectory/$OUTPUT_FILE
               rc=$?
               umount /tmp/profile/mnt/profiles
               if [ "$rc" = "0" ]; then
                  return 0
               fi
            fi
         else
            # Mount the flash device ??? 
            debug_echo "Export to boot device..."
            if mount --bind /nstation/profiles /tmp/profile/mnt/profiles; then
               debug_echo "cp -f $BASE/$PROFILE_FILE /tmp/profile/mnt/profiles/$ComponentDirectory/$OUTPUT_FILE"
               mkdir -p /tmp/profile/mnt/profiles/$ComponentDirectory
               cp -f $BASE/$PROFILE_FILE /tmp/profile/mnt/profiles/$ComponentDirectory/$OUTPUT_FILE
               rc=$?
               umount /tmp/profile/mnt/profiles
               if [ "$rc" = "0" ]; then
                  return 0
               fi
            fi
         fi
         ;;

      *):
         ;;
   esac

   debug_echo "Export failed."

   return 1
}



parse_options "$@"

retrieve_manifest_data $PROFILE_TYPE $MANIFEST_FILE

if [ ! -f $BASE/$PROFILE_FILE ]; then
   echo "Profile \"$BASE/$PROFILE_FILE\" not found."
   display_help
fi


upload_profile


