#!/bin/sh
#########################################################
#  Script to allow migration from SU-3 to R2.  This is also
#  used in part by tcm_restore for restoring SU-3 to R2
#
#  About Migration
#  ---------------
# The purpose of the migration program is to convert the configuration files from
# SU3 format to the new R2 profile component format.  All of the profile components
# created during migration are dependent on their definitions in the
# manifest.properties file (in the /nstation/default/profiles directory).
# Note, non user-configuration data in the home directory is not moved nor migrated.
#
#  Pre-Migration Setup
#  -------------------
# When SU4 is installed over an existing SU3 installation, the user and client
# configuration and home directories are moved from their following SU3 locations
# to their temporary holding place R2 locations:
#   /nstation/linux/root/*             ---->  /nstation/linux/IBM/migration/home/root
#   /nstation/linux/home/sys-default/* ---->  /nstation/linux/IBM/migration/home/root
#   /nstation/linux/home/<userid>/*    ---->  /nstation/linux/IBM/migration/home/<userid>
#   /nstation/machines/<mac>/tmp/home/KIOSK/*  ---->  /nstation/linux/IBM/migration/home/<mac>
#   /nstation/machines/<mac>/tmp/var/snmp/snmpd.local.trap  ---->  /nstation/linux/IBM/migration/home/<mac>
#
#  Post-Migration
#  --------------
# After Migration completes, the contents of the /nstation/linux/IBM/migration directory (but not the
# lock file) are tarred up in order to preserve the SU3 information but reduce the space requirement.
# The administrator can later remove this tar or untar it in the event they want migration to run again
# or they need to access any non-migrated user files.
# 
# All profile components created by the migration program are placed in the corresponding directory for
# their profile component type.  Specifically,
#    /nstation/profiles/user/<component-type>/
#    /nstation/profiles/machine/<component-type>/
#
# Two files are also created but not placed into a profile:
#               /nstation/machines/<mac>/snmp/snmpd.conf (trap information from snmpd.local.trap)
#               /nstation/machines/<mac>/snmp/snmpd.local.conf (inventory info from snmpd.local.trap)
#
# REMOVEFROMFINAL--NEED TO REMOVE BEFORE FINAL GA
#########################################################


###############################################################
#  Function:  migration_info()
#  Used to differentiate between echo used for writing to files
#  and echo used for error or informational messages
###############################################################
migration_info () {

  echo $@

}

migration_both () {

  echo $@
  echo $@ >> $migrationtmpdir/migration.output

}

check_for_debug () {

  if [ -n "$1" ]; then
    if [ $1 = "-debug" ]; then
      DEBUG=true
    fi
  fi

}

migration_debug () {
	
  if [ $DEBUG = "true" ]; then
    echo $@ >> $migrationtmpdir/migration.output
  fi

}

migration_output () {

    echo $@ >> $migrationtmpdir/migration.output

}

migration_check () {

  if [ -f $migrationtmpdir/migration.output ]; then
    echo $@ >> $migrationtmpdir/migration.output
  fi

}


echobar() {

  migration_debug "################################################"

}

echo_progress () {
   if [ "x$MODEL_TYPE" != "xN70" -a "$MIGRATIONMETHOD" = "migration" ]; then
      echo "$*" > /proc/progress
   fi

}



#####################################################################
#  Function:  parseargs()
#  Parse command line arguments
#####################################################################
parseargs() {

  while [ -n "$1" ]; do
    case $1 in
      "-debug"):
        DEBUG=true
        ;;
      "-serverpath"):
        shift
        if [ -n "$1" ]; then
          migrationserverdir=$1
        fi
        ;;
      "-backup_restore"):
        MIGRATIONMETHOD=backup
        ;;
      *):
        ;;
    esac

    shift
  done

}


#####################################################################
#  Function:  create_lock()
#  Create a lock file to stop other clients from migrating same files
#  Migration will occur on the first boot of a client to a particular
#  server.
#####################################################################
createlock() {

  userlock=.migrationlock
  #macaddress=`/sbin/ifconfig $adapter | grep HWaddr | awk '{print $NF}'`
  migration_debug $MACADDRESS
  migration_debug $migrationtmpdir/$userlock

  if [ ! -e $migrationtmpdir/$userlock ]; then 
    echo $MACADDRESS > $migrationtmpdir/$userlock
    chmod 400 $migrationtmpdir/$userlock
  else
    migration_info "No migration necessary.  Already commencing or completed."
    umountserver
    exit 1
  fi

  # Double check to make sure we are the ones who created the lock file
  if [ "$MACADDRESS" != "`cat $migrationtmpdir/$userlock`" ]; then
    migration_info "Did not create the lock file.  Exiting"
    umountserver
    exit 1
  fi

}


######################################################################
#  Function:  umountserver()
#  Create mount point to where install placed su3 files
######################################################################
umountserver() {

  umount $migrationprofilemounted >/dev/null 2>&1
  # umount $migrationmachinesmounted >/dev/null 2>&1

}

######################################################################
#  Function:  checkmount()
#  Confirms the server is up and the mount works
######################################################################
checkmount() {

  status="failure"

  #  mount -t $filetype $server:$migrationprofilemountdir $migrationprofilemountdir
  migration_debug "Running:  mount -t $filetype $server:$migrationprofilemountdir $migrationprofilemounted"
  if mount -t $filetype $server:$migrationprofilemountdir $migrationprofilemounted >/dev/null 2>&1; then
    status="success"
  fi

  # migration_debug "Running:  mount -t $filetype $server:$migrationmachinesmountdir $migrationmachinesmounted"
  # if mount -t $filetype $server:$migrationmachinesmountdir $migrationmachinesmounted >/dev/null 2>&1; then
  #  status="success"
  # fi

  return `test "$status" = "success"`

}

######################################################################
#  Function:  preparegroupfile()
#  Erbjorically update /etc/group with cifscommon gid.  For windows
#  authentication.
######################################################################
preparegroupfile() {
  echo "cifscommon::65001:" >> /etc/group
}


######################################################################
#  Function:  mountserver()
#  Create mount point to where install placed su4 profiles
######################################################################
mountserver() {

  migration_debug "Commencing Migration... server using:  $server"

  for mounts in $migrationprofilemounted $migrationmachinesmounted; do
    if [ ! -d $mounts ]; then
      migration_info "$mounts doesn't exist"
      mkdir -p $mounts
    fi

  done

  if ! checkmount; then
    migration_both "Mounts failed... EXITING MIGRATION"
    exit 2
  fi

}



######################################################################
#  Function:  prepareformigration()
#  Copy install files to proper places for migration.
#  For now this only consists of moving machineDir.IBM_NS.setup
######################################################################
prepareformigration() {

  if [ $MIGRATIONMETHOD = "migration" ]; then
    mv $migrationtmpdir/home/machineDir.IBM_NS.setup $migrationtmpdir/
  fi

}


######################################################################
#  Function:  migratesu3files()
# Copy su3 files to begin to prepare su4 files.
######################################################################
migratesu3files() {

  # Check to see if we have any files to migrate
  # We should actually check for a file instead since PEER_BOOT could
  # view a client as a server that needs to migrate.
  if [ ! -d $migrationtmpdir/home -a "$MIGRATIONMETHOD" = "migration" ]; then
    migration_check "$migrationtmpdir/home empty.  NOTHING NEEDS TO MIGRATE.  EXITING."
    exit 2
  fi

  #NEEDTODO::  Do we have to createlock for backup?
  if [ $MIGRATIONMETHOD = "migration" ]; then
    echo_progress "81 Migration commencing.  This could take a while."
    createlock
    preparegroupfile
    mountserver
    prepareformigration
  fi


  migration_output "Printout of $migrationtmpdir:"
  directorylisting=`ls -la $migrationtmpdir`
  migration_output "$directorylisting"

  # Will need to check to see if we are migrating users.
  # If yes by some file or environment variable then install
  # will provide such and we will continue accordingly.
  # if yes then users = all else users = root
  users=root
  usercount=1
  if [ -f $migrationtmpdir/migrate.home ] ; then
    users=`ls $migrationtmpdir/home`
#    usercount=`ls -1 /IBM/migration/home | wc -l | cut -d' ' -f7`
    usercount=`ls -1 /IBM/migration/home | wc -l`
  fi
  migration_debug "Compiling migration for $users"

  echobar
  for i in $users; do
    migration_output "cp -r $migrationsu3dir/$i/IBM/* $migrationsu3dir/$i/"
    cp -r $migrationsu3dir/$i/IBM/* $migrationsu3dir/$i/ 2>/dev/null
  done

}


#########################################################
#  Function:  dissectinfoblock()
#  Dissect IBMSetupInformationBlock
#  IBMSetupInformationBlock ---> IBMSetupMouseSettings
#				 IBMSetupAudioInfo
#				 IBMSetupPrintSpoolSize
#				 IBMSetupDefaultPrinter
#				 IBMSetupIcewmInfo
#				 IBMSetupServices
#				 IBMSetupInformationBlock
#########################################################
dissectinfoblock() {

  # Create the initial machine and user directories for the user
  if [ ! -d $migrationtmpdir/su4 -o ! -d $migrationtmpdir/su4/$userprefs \
    -o ! -d $migrationtmpdir/su4/$userprefs/machine \
    -o ! -d $migrationtmpdir/su4/$userprefs/user ]; then
    mkdir -p $migrationsu4dir/$userprefs/machine
    mkdir -p $migrationsu4dir/$userprefs/user
  fi

  if [ -f $currentsu3usrdir/IBMSetupInformationBlock ]; then

    mousesettings=`cat $currentsu3usrdir/IBMSetupInformationBlock | grep LeftHandedMouse=`
    audioinfo=`cat $currentsu3usrdir/IBMSetupInformationBlock | grep soundmaster=`
    printspoolsize=`cat $currentsu3usrdir/IBMSetupInformationBlock | grep PrintSpoolSize=`
    defaultprinter=`cat $currentsu3usrdir/IBMSetupInformationBlock | grep defaultprinter`
    icewminfo1=`cat $currentsu3usrdir/IBMSetupInformationBlock | grep ICEWM_ICA_KEY_REDUCED=`
    icewminfo2=`cat $currentsu3usrdir/IBMSetupInformationBlock | grep screensaver`
    services=`cat $currentsu3usrdir/IBMSetupInformationBlock | grep PEERBOOT_ENABLE=`
    setupinfoblock=`cat $currentsu3usrdir/IBMSetupInformationBlock | grep showlowmemdialog`

    migration_debug "##########################################################"
    migration_debug "Checking for files read from IBMSetupInformationBlock:"
    migration_debug "mouse:  $mousesettings"
    migration_debug "audio:  $audioinfo"
    migration_debug "spool:  $printspoolsize"
    migration_debug "default: $defaultprinter"
    migration_debug "ice1:  $icewminfo1"
    migration_debug "ice2:  $icewminfo2"
    migration_debug "services:  $services"
    migration_debug "info:  $setupinfoblock"
    migration_debug "Done reading variables"
    migration_debug "##########################################################"
    migration_debug ""

    createfilesfrominfoblock
  fi

}


##############################################################
#  Function:  createfilesfrominfoblock
#  Create files from IBMSetupInformationBlock
#  displaycolor          -------> <defunct : nvram setting>
#  screensaver           -------> IBMSetupIcewmInfo
#  soundmaster           -------> IBMSetupAudioInfo
#  clientserverconfigure -------> Not begin used in su4
#  PEERBOOT_ENABLE       -------> IBMSetupServices
#  ICEWM_ICA_KEY_REDUCED -------> IBMSetupIcewmInfo
#  LeftHandedMouse       -------> IBMSetupMouseSettings
#  PrintSpoolSize        -------> IBMSetupPrintSpoolSize
#  showlowmemdialog      -------> IBMSetupInformationBlock
#  defaultprinter        -------> IBMSetupDefaultPrinter
##############################################################
createfilesfrominfoblock() {
  echo $mousesettings > $migrationsu4userdir/IBMSetupMouseSettings
  echo $audioinfo > $migrationsu4userdir/IBMSetupAudioInfo
  echo $printspoolsize > $migrationsu4machinedir/IBMSetupPrintSpoolSize
  echo $defaultprinter > $migrationsu4machinedir/IBMSetupDefaultPrinter
  echo $icewminfo1 > $migrationsu4userdir/IBMSetupIcewmInfo
  echo $icewminfo2 | awk '{printf $3"\n" $2"\n" $1"\n"}' >> $migrationsu4userdir/IBMSetupIcewmInfo

  if grep DesktopBackgroundImage $currentsu3rootdir/IBMSetupDesktopConfig > /dev/null
  then
    grep -v "DesktopBackgroundImage" $currentsu3rootdir/IBMSetupDesktopConfig > /tmp/DesktopConfig
    mv /tmp/DesktopConfig $currentsu3rootdir/IBMSetupDesktopConfig
  fi
 
  if grep DesktopBackgroundImage $currentsu3usrdir/IBMSetupDesktopConfig > /dev/null
  then
    grep -v "DesktopBackgroundImage" $currentsu3usrdir/IBMSetupDesktopConfig > /tmp/DesktopConfig
    mv /tmp/DesktopConfig $currentsu3usrdir/IBMSetupDesktopConfig
  fi

  if [ -f $migrationsu4userdir/IBMSetupIcewmInfo ]; then
    grep -v "DesktopBackgroundImageName" $migrationsu4userdir/IBMSetupIcewmInfo > /tmp/IcewmInfo
    mv /tmp/IcewmInfo $migrationsu4userdir/IBMSetupIcewmInfo
    echo "DesktopBackgroundImageName=\"/tmp/profile/user/wm_background.xpm\"" >> $migrationsu4userdir/IBMSetupIcewmInfo
  fi
 
  echo $services > $migrationsu4machinedir/IBMSetupServices
  echo $setupinfoblock > $migrationsu4userdir/IBMSetupInformationBlock

}


##########################################################
#  Function:  dissectnetscapefiles()
#  Dissect preferences.js
#  Change browser.startup.license_accepted from
#  4.77 to 4.78.
##########################################################
dissectnetscapefiles() {

  preferencesfile=$currentsu3usrdir/.netscape/preferences.js
  if [ -f $preferencesfile ] ; then
    cat $preferencesfile | sed s/4.77/4.78/ >> $preferencesfile.new
    mv -f $preferencesfile.new $preferencesfile
  fi
  cat $preferencesfile | 
  {
    grep -v "browser.bookmark_file" |
    grep -v "browser.cache.directory" |
    grep -v "browser.history_file" |
    grep -v "browser.sarcache.directory" |
    grep -v "browser.user_history_file"
  } > $preferencesfile.su4  
  {
    echo "user_pref(\"browser.bookmark_file\", \"/tmp/profile/user/home/.netscape/bookmarks.html\");"
    echo "user_pref(\"browser.cache.directory\", \"/tmp/profile/user/.netscape/cache\");"
    echo "user_pref(\"browser.history_file\", \"/tmp/profile/user/.netscape/history.db\");"
    echo "user_pref(\"browser.sarcache.directory\", \"/tmp/profile/user/.netscape/archive\");"
    echo "user_pref(\"browser.user_history_file\", \"/tmp/profile/user/.netscape/history.list\");"
  } >> $preferencesfile.su4
  mv -f $preferencesfile.su4 $preferencesfile  
}


##########################################################
#  Function:  dissectsnmp()
#  Dissect snmpd.local.trap
#  snmpd.local.trap ---> snmpd.conf (trap data)
#			 snmpd.local.conf (inventory data)
##########################################################
dissectsnmp() {

  snmpdfile=$currentsu3usrdir/snmpd.local.trap

  if [ -f $snmpdfile ]; then

    # Settings for snmpd.conf in SU4
    for trapsettings in authtrapenable trapcommunity trapsink; do
      trapsetting=`cat $snmpdfile | grep $trapsettings`

      if [ -n "$trapsetting" ]; then
        grep $trapsettings $snmpdfile >> $migrationsu4dir/snmpd.conf
        #echo $trapsetting >> $migrationsu4dir/snmpd.conf
      fi
    done

    # Settings for snmpd.local.conf in SU4
    for inventorysettings in aliasname syslocation serialnumber \
                             syscontact imagename imagedate ; do
      inventorysetting=`cat $snmpdfile | grep $inventorysettings`
      if [ -n "$inventorysetting" ]; then
        echo $inventorysetting >> $migrationsu4dir/snmpd.local.conf
      fi

    done

    # userdefinedlabelX and userdefinedvalueX could have up to 4 entries.
    for userdefinedsettings in userdefinedlabel userdefinedvalue; do
      for levels in 1 2 3 4; do
        userdefinedsetting=`cat $snmpdfile | grep $userdefinedsettings$levels`

        if [ -n "$userdefinedsetting" ]; then
          echo $userdefinedsetting >> $migrationsu4dir/snmpd.local.conf
        fi

      done

    done

  fi


  # Copy over to /nstation/machines/<mac>/snmp dir
  if [ ! -d $migrationmachinesmounted/$userprefs/snmp ]; then
    mkdir -p $migrationmachinesmounted/$userprefs/snmp
  fi

  cp $migrationsu4dir/snmpd.conf $migrationmachinesmounted/$userprefs/snmp/ >/dev/null 2>&1
  cp $migrationsu4dir/snmpd.local.conf $migrationmachinesmounted/$userprefs/snmp/>/dev/null 2>&1
}


##############################################################
#  Function:  createIBMSetupDnsServerfile
#  Create the IBMSetupDnsServer file
##############################################################
createIBMSetupDnsServerfile() {

  # IBMSetupDnsServer: (nameserver=ns) ns+DHCP | ns+static | no ns
  #         usedns                      True       True      False
  #         dns_radio_button              0          1         2
  #         host_radio_button             0          0         1
  #         nameserver                  none       ns1/2      none
  #

    # default setting for Nameserver and DHCP addressing in use
    _dns=True
    _dns_radio_button=0
    _host_radio_button=0

    # No Nameserver in use
    if [ \( -z "$NAMESERVER1" -o "$NAMESERVER1" = "0.0.0.0" \) -a \
         \( -z "$NAMESERVER2" -o "$NAMESERVER2" = "0.0.0.0" \) ]; then
      
        _dns=False
        _dns_radio_button=2
        _host_radio_button=1
    else
      # Check nvram to see if we are using DHCP
      # For now CONFIGMODE is not working so use /tmp/snmp/93 instead
      ADDRESSING=`cat /tmp/snmp/93`
      if [ "$ADDRESSING" = "static" ]; then
        #Nameserver and Static IP Addressing in use
        _dns_radio_button=1
        for nameservers in nameserver1=$NAMESERVER1 \
                           nameserver2=$NAMESERVER2 ; do
          echo $nameservers >> $migrationsu4machinedir/IBMSetupDnsServers
        done
      fi
    fi
        
    #What about domainname=?
    for values in usedns=$_dns \
                  dns_radio_button=$_dns_radio_button \
                  host_radio_button=$_host_radio_button ; do
      echo $values >> $migrationsu4machinedir/IBMSetupDnsServers
    done

}

##############################################################
#  Function:  createneededfiles
#  Create the authentication and setuplogin profiles
#  Also move over printcap from etc
##############################################################
createneededfiles() {

# Delete IBMSetupAuthentication and IBMSetupLogin files since the only
# reason they should be here is a failed migration.
for files in IBMSetupAuthentication IBMSetupLogin ; do
  rm -f $migrationsu4machinedir/$files
done

# Move over printcap file from /IBM/migration/etc to su4userdir
# This will only be done for root since on su3 that was the only
# place this would have been located (/etc/printcap)
if [ "$userprefs" = "root" ]; then
  cp $migrationtmpdir/etc/printcap $migrationsu4machinedir/printcap >/dev/null 2>&1
  rm -f /tmp/.printcap.orig
  if [ -f $migrationsu4machinedir/printcap ]; then
        sed -e 's/:\\//' -e 's/:$//g' $migrationsu4machinedir/printcap > /tmp/.printcap.orig
        cp /tmp/.printcap.orig $migrationsu4machinedir/printcap
  fi
  if [ -f $migrationsu4machinedir/printcap ]; then
        rm -f /tmp/.printcap
        cat $migrationsu4machinedir/printcap | while IFS= read i
        do
                # look for all lines that aren't a comment
                line=`echo $i |  sed -n '/^[^#]/p'`
                if [ "$line" ] ; then

                        # check for input filter line... this is last line in an entry
                        ifline=`echo $line | sed -n '/.*:if/p'`
                        if [ "$ifline" = "" ] ; then
                                echo "$i:\\" >> /tmp/.printcap
                        else
                                echo "$i:" >> /tmp/.printcap
                        fi
                else
                        echo "$i" >> /tmp/.printcap
                fi
        done

        cp /tmp/.printcap $migrationsu4machinedir/printcap
        rm -f /tmp/.printcap
  fi
  mkdir -p $migrationsu4machinedir/spool
  cp -R /tmp/tcm/restore/root/lpd $migrationsu4machinedir/spool
  dirlist=`ls -ld $migrationsu4machinedir/spool/lpd/* | awk {'print $NF'}`
  for i in $dirlist; do
    cp /usr/lib/rhs/rhs-printfilters/master-filter /$i/filter
  done
fi

# Create IBMSetupAuthentication file according to authentication
# Check how su3 was installed by checking machineDir.IBM_NS.setup
# During backup, the above file will not exist.
# If doing backup, then need to create KIOSK version
if [ "$MIGRATIONMETHOD" = "backup" ]; then
  passwd_size=`ls -l /IBM/migration/etc/passwd | awk '{print $5}'`
  passwd_last=`tail -1 /IBM/migration/etc/passwd | awk -F: '{print $1}'`
  if [ "$passwd_size" = "667" -o "$passwd_last" = "KIOSK" ]; then
      installmode=KIOSK
  else
      installmode=NONKIOSK
  fi
else
  installmode=`grep MACHINE_TEMPLATE /IBM/migration/machineDir.IBM_NS.setup | cut -d= -f2 | cut -d_ -f1`
fi

migration_output "installmode used $installmode"

if [ "$installmode" = "NONKIOSK" ]; then

  echo "AuthenticatedLoginType=0" >> $migrationsu4machinedir/IBMSetupAuthentication
  echo "HomeDirType=0" >> $migrationsu4machinedir/IBMSetupAuthentication

  # Create IBMSetupLogin file
  for values in AutomaticLoginEnable='"false"' AllowRoot='"true"'; do
    echo $values >> $migrationsu4machinedir/IBMSetupLogin
  done
elif [ "$installmode" = "KIOSK" ]; then
  # Create IBMSetupLogin file
  for values in AutomaticLoginEnable='"true"' AllowRoot='"true"' HomeDirPersistent='"true"' AutomaticLoginProfileUser='"KIOSK"'; do
    echo $values >> $migrationsu4machinedir/IBMSetupLogin
  done
fi

  # For root only create IBMSetupDnsServer and modify IBMSetupTimeServer
  if [ $userprefs = "root" ]; then

#    createIBMSetupDnsServerfile
    if [ -f $migrationsu3rootdir/IBMSetupTimeServerInfo ]; then
      mv $migrationsu3rootdir/IBMSetupTimeServerInfo $migrationsu3rootdir/IBMSetupTimeServerInfo.orig
      for timesettings in timeserver timeprotocol; do
        timesetting=`grep $timesettings $migrationsu3rootdir/IBMSetupTimeServerInfo.orig`
        if [ -n "$timesetting" ]; then
          echo "primary_$timesetting" >> $migrationsu4machinedir/IBMSetupTimeServerInfo
        fi
      done

      timezone=`grep timezone $migrationsu3rootdir/IBMSetupTimeServerInfo.orig`
      if [ -n "$timezone" ]; then
        echo $timezone >> $migrationsu4machinedir/IBMSetupTimeServerInfo
      fi

#      mv $migrationsu3rootdir/IBMSetupTimeServerInfo.orig $migrationsu3rootdir/IBMSetupTimeServerInfo
    # If IBMSetupTimeServerInfo does not exist then create default with GMT
    else
      echo "timezone=GMT" >> $migrationsu4machinedir/IBMSetupTimeServerInfo
    fi
  fi
}

##############################################################
#  Function:  confirmfiles
#  Checks for existence of files for a profile.
#  If no files then will not create a zip file.
##############################################################
confirmfiles() {

  create="no"
  currentdir=$PWD

  migration_debug "Confirm files for profile $1"
  filestoconfirm=`grep ^$1.Files $manifestproperties | cut -d= -f2`
  migration_debug "Filestoconfirm:  $filestoconfirm"
  type=`grep ^$1.Type $manifestproperties | cut -d= -f2`
  migration_debug "Type for Filestoconfirm $type"

  cd $currentsu4usrdir/$type

  for i in $filestoconfirm; do
    if [ "$i" = "home/.ICAClient" ]; then
       i=".ICAClient/*"
    fi
    case $type in
      "user"):
        migration_debug "File $i ("$type") checking $migrationsu4userdir/$i"

        if [ -e $migrationsu4userdir/$i ]; then
          migration_debug "Doing create for user profile $1"
          create="yes"
          break
        fi
        ;;
      "machine"):
        migration_debug "File $i ("$type") checking $migrationsu4machinedir/$i"

        if [ -e $migrationsu4machinedir/$i ]; then
          migration_debug "Doing create for user profile $1"
          create="yes"
          break
        fi
        ;;
    esac
  done

  cd $currentdir

}


##############################################################
#  Function:  checkforfile
#  Checks for existence of file in user and if not there then
#  checks default (root).  This is ONLY done for desktop!
##############################################################
checkforfile() {

  currentcheckdir=$PWD
  cd $currentsu3usrdir
  migration_debug "Changing from $currentcheckdir to $currentsu3usrdir"
  checkedfile="none"

  # Check userlevel first then check default
  for checkfiles in $1 ; do
    migration_debug "Check $checkfiles in $currentsu3usrdir/$1 for $2 (`dirname $checkfiles`/)"

    # Broken for new release since they no longer use * for .ICAClient dir
    # Now just check to see if exists and assume directories will not be empty.
    if [ -e $checkfiles ]; then
      checkedfile="user"
    elif [ $2 = "desktop" ] ; then
      cd $currentsu3rootdir
      for checkrootfiles in $checkfiles ; do

        if [ -e $currentsu3rootdir/$checkrootfiles ]; then
          migration_debug "Checked as default in $currentsu3rootdir/$checkrootfiles for $2"
          checkedfile="default"
        fi
      done
      cd $currentsu3usrdir
    fi
  done

  cd $currentcheckdir
}

##############################################################
#  Function:  copyuserfiles
#  Copy the other requested files that are needed
#  Files included or copied based on 
##############################################################
copyuserfiles() {

for files in $usersu4files ; do
  su3format=`echo $files | sed s/home'\/'//`
  checkforfile $su3format $1
  migration_debug "Checkforfile in user returned $checkedfile"

  case $checkedfile in
    "user"):
       mkdir -p $migrationsu4userdir/`dirname $files`
       migration_debug "current dir:  $currentsu3usrdir/$su3format to $migrationsu4userdir/$files/"
       cp -r $currentsu3usrdir/$su3format $migrationsu4userdir/`dirname $files`/ 
    ;;
    "default"):
       mkdir -p $migrationsu4userdir/`dirname $files`
       migration_debug "current dir:  $migrationsu3rootdir/$su3format to $migrationsu4userdir/`dirname $files`/"
       cp -r $migrationsu3rootdir/$su3format $migrationsu4userdir/`dirname $files`/ 
    ;;
    "none"):
       migration_debug "$files doesn't exist for user nor default"
    ;;
  esac
done

}


##############################################################
#  Function:  copymachinefiles
#  Copy the other requested files that are needed
#  Files included or copied based on 
##############################################################
copymachinefiles() {

for files in $machinesu4files ; do
  su3format=`echo $files | sed s/home'\/'//`

  checkforfile $su3format $1
  migration_debug "Checkforfile returns $checkedfile"

  case $checkedfile in
    "user"):
       mkdir -p $migrationsu4machinedir/`dirname $files`
       migration_debug "current dir:  $currentsu3usrdir/$su3format to $migrationsu4machinedir/$files"
       cp -r $currentsu3usrdir/$su3format $migrationsu4machinedir/`dirname $files`/
    ;;
    "default"):
       mkdir -p $migrationsu4machinedir/`dirname $files`
       migration_debug "current dir:  $migrationsu3rootdir/$su3format to $migrationsu4machinedir/`dirname $files`"
       cp -r $migrationsu3rootdir/$su3format $migrationsu4machinedir/`dirname $files`/
    ;;
    "none"):
       migration_debug "$files doesn't exist for user nor default"
    ;;
  esac
done

}


##############################################################
#  Function:  copyfixedfiles
#  Copy the other requested files that are needed
#  Files included or copied based on 
##############################################################
copyfixedfiles() {

# Generates files from manifest.properties
# First use the manifest.properties to copy over files
#files=`grep '\.Files' $manifestproperties | cut -d= -f2 | sed s/home'\/'//\g`
usertypes=`grep Type=user $manifestproperties | cut -d. -f1`
machinetypes=`grep Type=machine $manifestproperties | cut -d. -f1`

migration_debug "usertypes:  $usertypes"
migration_debug "###########################"
migration_debug "machinetypes:  $machinetypes"
migration_debug "###########################"


for i in $usertypes ; do
  usersu4files=`grep ^$i.Files $manifestproperties | cut -d= -f2`
  migration_debug "############ Printing out files:  users ($i) ####################"
  migration_debug $usersu4files
  migration_debug "#################################################################"

  copyuserfiles $i
done

for i in $machinetypes ; do
  machinesu4files=`grep ^$i.Files $manifestproperties | cut -d= -f2`
  migration_debug "############ Printing out files:  machines ($i) #################"
  migration_debug $machinesu4files
  migration_debug "#################################################################"

  copymachinefiles $i
done

}

###############################################################
#  Function:  getprofileservers
#  Gets the two profile servers and also uses the bootserver
#  as a backup
###############################################################
getprofileservers() {

   if [ ! -f /tmp/snmp/82 ]; then
      /IBM/bin/readNvram 82
   fi
   nvram_82=`cat /tmp/snmp/82`

   if [ "$nvram_82" != "null0.0.0.0" ]; then
      protocol=`echo $nvram_82 | cut -d: -f1`
      path=`echo $nvram_82 | sed s/^.*:\\\/\\\/[^\\/]*//`

      if [ "$protocol" = "http" -a "$path" = "/TCMServlet" ]; then
         PROFILE_URL_1="$nvram_82/GetProfiles?${user_option}&IP=${ip}&MAC=${mac}"
      else
         PROFILE_URL_1=$nvram_82
      fi
   fi

   if [ ! -f /tmp/snmp/83 ]; then
      /IBM/bin/readNvram 83
   fi
   nvram_83=`cat /tmp/snmp/83`

   if [ "$nvram_83" != "null0.0.0.0" ]; then
      protocol=`echo $nvram_83 | cut -d: -f1`
      path=`echo $nvram_83 | sed s/^.*:\\\/\\\/[^\\/]*//`

      if [ "$protocol" = "http" -a "$path" = "/TCMServlet" ]; then
         PROFILE_URL_2="$nvram_83/GetProfiles?${user_option}&IP=${ip}&MAC=${mac}"
      else
         PROFILE_URL_2=$nvram_83
      fi
   fi

   #For backup use the BootServer
   PROFILE_BOOT_SERVER="$server"
}

###############################################################
#  Function:  createprofile
#  Create the profiles 
###############################################################
createprofile() {

  profiles=$1
  confirmfiles $profiles

  if [ $create = "yes" ]; then
    #Need to replace periods with underscores
    userprefs=`echo $userprefs | sed s/\\\./_/g`
    migration_output "Creating profile:  $profile_dir/create_profile.sh -b $currentsu4usrdir -m $manifestproperties $profiles "migratedrootsettings""

    zipfile=`$profile_dir/create_profile.sh -b $currentsu4usrdir -m $manifestproperties $profiles "migratedrootsettings"`

    migration_output "$zipfile:  $userprefs DOING:  $profile_dir/export_profile.sh --debug -b $migrationprofiledir -m $manifestproperties -f <user or default if root> $zipfile file:///tmp/profile/mnt/profiles"

    if [ $userprefs = "root" ]; then
      $profile_dir/export_profile.sh -b $migrationprofiledir -m $manifestproperties -f DEFAULT $zipfile file:///tmp/profile/mnt/profiles
    else
      $profile_dir/export_profile.sh -b $migrationprofiledir -m $manifestproperties -f $userprefs $zipfile file:///tmp/profile/mnt/profiles
    fi

  else
    migration_output "Will not create $profiles profile since no files are available"
  fi

}


###############################################################
#  Function:  prepareprofile
#  Setup creation of profiles 
###############################################################
prepareprofile() {

if [ $userprefs = "root" ]; then
  for rootonlyprofiles in printers-machine printers-user servers mountpoints ; do

    createprofile $rootonlyprofiles

  done
fi

for allprofiles in desktop icewmprefs ica netscape 3270 5250 audio \
		mouse seriald authentication mountpoints  \
		environmentvariables services ; do

  createprofile $allprofiles

done

}


###############################################################
#  Function:  cleanupsu3files
#  Cleans up the su3 dir that was migrated over by install
#  Tar up all of $serverBaseDir/IBM/migration/home and remove
#  the corresponding directory after
###############################################################
cleanupsu3files() {

  currentdir=$PWD

  cd /IBM/migration/

  if [ $MIGRATIONMETHOD = "migration" ]; then
    #if [ ! -d /IBM/backup ]; then
    #  mkdir /IBM/backup
    #fi

    mv machineDir.IBM_NS.setup home/ >/dev/null 2>&1
    # REMOVEFROMFINAL
    # tar -czvf /IBM/backup/SU4UserFiles.tgz migrate.home home etc >/dev/null 2>&1
    #  rm machineDir.IBM_NS.setup
    rm migrate.home > /dev/null 2>&1
  fi

  rm -rf etc
  rm -rf home
  rm -rf su3
  rm -rf su4

  cd $currentdir
}


###############################################################
#  Function:  tarupprofiles
#  Tar up the profiles and place into /nstation/linux/IBM/migration
###############################################################
tarupprofiles() {
  currentdir=$PWD
  cd $migrationprofilemounted

  tar -czvf SU4MigrationProfiles.tgz machine user >/dev/null 2>&1
  mv SU4MigrationProfiles.tgz $migrationtmpdir

  cd $currentdir
}

#######################################################
#  Beginning of migration script.
#######################################################

migrationtmpdir=/IBM/migration
migrationsu3dir=$migrationtmpdir/home
migrationsu4dir=$migrationtmpdir/su4
migrationserverdir=$serverBaseDir/IBM/migration
migrationsu3rootdir=$migrationsu3dir/root
migrationprofilemountdir=`dirname $serverBaseDir`/profiles
migrationprofilemounted=/tmp/profile/mnt/profiles
migrationmachinesmountdir=`dirname $serverBaseDir`/machines
migrationmachinesmounted=/tmp/profile/mnt/machines
migrationprofiledir=/tmp/profile/generated

# This has been changed due to blue devil
manifestproperties=/IBM/manifest.properties
profile_dir=/IBM/bin

MIGRATIONMETHOD=migration
DEBUG=false
filetype=nfs
currentcount=0

# Let's save off the migration.output files for restore.
date=`date +%X | sed s/:/./g | awk '{printf $1}'` 
if [ -f $migrationtmpdir/migration.output ]; then
  migration_output "mv $migrationtmpdir/migration.output $migrationtmpdir/migration.output.$date"
  mv $migrationtmpdir/migration.output $migrationtmpdir/migration.output.$date
fi

parseargs "$@"
#mountserver

#########################################################
#  For each user including root, assemble files
#########################################################
migratesu3files

migration_both "############ BEGINNING OF MIGRATING USER CONFIGURATIONS ############"
migration_both "This could take some time depending on the number of users being migrated..."
echo_progress "81 Migration started.  Reboot will occur when done."
sleep 2

for userprefs in $users; do

  # Keep track of number of users migrating
  currentcount=`expr $currentcount + 1`
  usercount=`expr $usercount + 0`

  echo_progress "81 Migrating user $userprefs ($currentcount of $usercount)"
  migration_both "Migrating user $userprefs ($currentcount of $usercount)..."
  currentsu3usrdir=$migrationsu3dir/$userprefs
  currentsu4usrdir=$migrationsu4dir/$userprefs
  currentsu3rootdir=$migrationsu3dir/root
  migrationsu4machinedir=$currentsu4usrdir/machine
  migrationsu4userdir=$currentsu4usrdir/user

  migration_output "Current settings for $userprefs:  $currentsu3usrdir $currentsu4usrdir"
  dissectinfoblock
  dissectsnmp
  dissectnetscapefiles
  createneededfiles
  copyfixedfiles
  prepareprofile

done

#tarupprofiles
cleanupsu3files

if [ $MIGRATIONMETHOD = "migration" ]; then
  umountserver
fi

migration_both "############ MIGRATION COMPLETE ############"

# We need to reboot the networkstation since not all profiles are being picked up
# at this time since migration is happening after IBMSetup already uses some profiles.
if [ $MIGRATIONMETHOD = "migration" ]; then
  echo_progress "81 MIGRATION COMPLETE.  WORKSTATION WILL REBOOT NOW."
  sleep 5
  /IBM/bin/reboot
fi
